import com.kelvinluck.flashr.core.Person;
import com.kelvinluck.util.LogWrapper;
/**
* Class: com.kelvinluck.flashr.core.Comment
* Class to describe a comment on a photo on Flickr.
* 
* TODO:
* Should a comment somehow be associated with it's author? So that you had Person.getComments or something?
* Probably doesn't make sense because there is no current way to get all of a user's comments without 
* checking the comments on every photo...
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck < kelvin at kelvinluck dot com >
*
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.Comment 
{
	
	/**
	* Variable: id
	* This comment's id.
	**/
	public var id:String;
	/**
	* Variable: author
	* A reference to the <Person> who made this comment.
	**/
	public var author:Person;
	/**
	* Variable: created
	* The date this comment was made on.
	**/
	public var created:Date;
	/**
	* Variable: url
	* This permalink to this comment.
	**/
	public var url:String;
	/**
	* Variable: comment
	* The actual contents of this comment.
	**/
	public var comment:String;
	
	
	/**
	* Variable: _comments
	* A private static Object containing Comment objects. Used by <getComment> to insure that only 
	* one Comment is created for each comment ID returned from flickr.com
	**/
	private static var _comments:Object = {};
	
	/**
	* Function: Comment
	* Constructor function	-	Creates a new Comment object.
	* 
	* Should be called via <Comment.getComment>
	**/
	private function Comment(id:String)
	{
		LogWrapper.getLog().debug("com.kelvinluck.flashr.core.Comment instance created - id="+id);
		this.id = id;
	}
	
	
	/**
	* Function: getComment
	* Get's a Comment object for the given comment ID.
	* 
	* Consults <_comments> to make sure that only one Comment instance is created for each
	* comment ID from flickr.com
	* 
	* Parameters:
	* commentId				-	The id of the Comment you want to get
	* 
	* Returns:
	* The <Comment> object for the given id
	**/
	public static function getComment(commentId:String):Comment
	{
		if (_comments[commentId] == undefined) {
			_comments[commentId] = new Comment(commentId);
		}
		return _comments[commentId];
	}
	
	function toString():String
	{
		return "[Object com.kelvinluck.flashr.core.Comment - "+id+"]";
	}
}