﻿import mx.events.EventDispatcher; // TODO: remove reliance on MM's EventDispatcher
import com.dynamicflash.utils.Delegate;
import com.suite75.ascrypt.MD5;
import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.util.DateFormat;
import com.kelvinluck.flashr.core.Photo;
import com.kelvinluck.flashr.core.Person;
import com.kelvinluck.flashr.core.Group;
import com.kelvinluck.flashr.core.Photoset;
import com.kelvinluck.flashr.core.ExifData;
import com.kelvinluck.flashr.core.Note;
import com.kelvinluck.flashr.core.Tag;
import com.kelvinluck.flashr.core.PhotoSize;
import com.kelvinluck.flashr.core.ResultsSet;
import com.kelvinluck.flashr.core.FlashrRequest;
import com.kelvinluck.flashr.core.FlashrRequestQueue;
import com.kelvinluck.flashr.core.FlashrResponseEvent;
import com.kelvinluck.flashr.core.FlashrCache;
import com.kelvinluck.flashr.core.FlashrResponseHistory;
import com.kelvinluck.flashr.core.Comment;

/**
* Class: com.kelvinluck.flashr.core.Flashr
* 
* 
* Class to provide an interface to the Flickr API
* 
* 
* This class in combination with <FlashrResponse>
* provides a convienient way to interact with the Flickr API from
* AS 2. The implementation should abstract away any processing of XML
* and allow for type checking in the functions you override in 
* FlashrResponse. The idea is to make flash code which interacts
* with the Flickr API quicker and easier to write.
* 
* 
* This code is still beta so feedback is greatfully received :)
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Usage:
* [TODO]
*  
* See Also:
* <FlashrResponse>
* 
* <http://www.flickr.com/services/api/>
* 
* Version:
* 0.5.0
* 
* Author:
* Kelvin Luck < kelvin at kelvinluck dot com >
* 
* Project Website:
* <http://kelvinluck.com/projects/flashr-a-flickr-api-wrapper-for-flash>
**/

class com.kelvinluck.flashr.core.Flashr {
	
	private static var _flashr:Flashr;
	/**
	* Variable: apiKey
	* 
	* Your Flickr API Key. None of the method calls will work without this.
	**/
	private var _apiKey:String;
	/**
	* Variable: _REST_ENDPOINT
	* 
	* The place for all API calls to connect to.
	* The default value of connecting directly to the flickr.com server will only
	* work if a relevant entry is added to their crossdomain.xml file
	* otherwise set the value to the location of a proxy script on your
	* server.
	**/
	private var _REST_ENDPOINT:String;
	
	/**
	* Variable: _AUTH_ENDPOINT
	* 
	* The base path you go to when you want to authenticate on flickr.com
	**/
	private var _AUTH_ENDPOINT:String = "http://flickr.com/services/auth/";
	
	// used internally to lock the object as only one request can be made at a time
	private var awaitingResponse:Boolean;
	
	private var _restXML:XML;	
	private var _currentRequest:FlashrRequest;
	
	/**
	* Variable: STATUS_BUSY
	* The API is already waiting for the response to a method call so cannot make another call
	**/
	public static var STATUS_BUSY:Number = 104;
	public static var STATUS_BUSY_DESCRIPTION:String = "The API is already waiting for the response to a method call so cannot make another call";
	/**
	* Variable: STATUS_ERROR_CONNECTING
	* Could not connect to the server (e.g. no internet connection, server down, security restrictions)
	**/
	public static var STATUS_ERROR_CONNECTING:Number = 102;
	public static var STATUS_ERROR_CONNECTING_DESCRIPTION:String = "Could not connect to the server (e.g. no internet connection, server down, security restrictions)";
	/**
	* Variable: STATUS_INVALID_XML
	* The XML returned by the server wasn't in a &lt;rsp /&gt; tag.
	**/
	public static var STATUS_INVALID_XML:Number = 101;
	public static var STATUS_INVALID_XML_DESCRIPTION:String = "Unexpected XML Format returned by server";
	
	/**
	* Variable: STATUS_USER_NOT_LOGGED_IN
	* The user wasn't logged in and tried to access a method that requires authentication
	* (returned from flickr.com)
	**/
	public static var STATUS_USER_NOT_LOGGED_IN:Number = 99;
	/**
	* Variable: STATUS_INVALID_API_KEY
	* The API key sent to flickr was invalid
	* (returned from flickr.com)
	**/
	public static var STATUS_INVALID_API_KEY:Number = 100;
	
	/**
	* Variable: STATUS_OK
	* There was no errors and valid XML was returned.
	**/
	public static var STATUS_OK:Number = 1000;
	
	/**
	* Variable: PERM_NOBODY
	* 
	* A constant representing permissions for nobody.
	* 
	* See Also:
	* <photosSetPerms>
	**/
	public static var PERM_NOBODY:Number = 0;
	/**
	* Variable: PERM_FRIENDS
	* 
	* A constant representing permissions for friends and family.
	* 
	* See Also:
	* <photosSetPerms>
	**/
	public static var PERM_FRIENDS:Number = 1;
	/**
	* Variable: PERM_CONTACTS
	* 
	* A constant representing permissions for contacts.
	* 
	* See Also:
	* <photosSetPerms>
	**/
	public static var PERM_CONTACTS:Number = 2;
	/**
	* Variable: PERM_EVERYBODY
	* 
	* A constant representing permissions for everybody.
	* 
	* See Also:
	* <photosSetPerms>
	**/
	public static var PERM_EVERYBODY:Number = 3;
	
	/**
	* Variable: _secret
	* 
	* Private variable to store this app's shared secret for use with the
	* authentication API.
	* 
	* See Also:
	* <authLogin>
	* 
	* <http://www.flickr.com/services/api/auth.spec.html>
	* 
	* <http://www.flickr.com/services/api/registered_keys.gne>
	**/
	private var _secret:String;
	
	/**
	* Variable: _authToken
	* 
	* Private variable which stores the token generated by a successfull
	* authorisation.
	**/
	private var _authToken:String = "";
	function get authToken():String
	{
		return _authToken;
	}
	function set authToken(_authToken:String):Void
	{
		this._authToken = _authToken;
	}
	
	/**
	* Variable: _authFrob
	* 
	* Private variable which is used to store the frob which is used for
	* the first part of the authentication process.
	**/
	private var _authFrob:String;
	
	private var _authIsLoggingIn:Boolean;
	
	/**
	* Variable: authPerms
	* 
	* A variable describing the permissions the currently auth'd user
	* has within this app. It will be "none" until the user has been
	* logged in and then will become "read", "write" or "delete". You can
	* check whether authPerms == "none" if you want to find out if the
	* user is currently logged in before checking who they are with <authUser>
	* 
	* See Also:
	* <authLogin>
	* 
	* <authUser>
	**/
	var authPerms:String = "none";
	
	/**
	* Variable: authUser
	* 
	* A <Person> object for the currently auth'd user (if there is one).
	* Empty if there is no currently auth'd user.
	* 
	* See Also:
	* <authLogin>
	* 
	* <authPerms>
	**/
	var authUser:Person;
	
	/**
	* Variable: cacheQueries
	* Sets whether the results of queries to flickr.com should be cached. The advantage of cached queries
	* is (much) quicker responses for repeated calls. The disadvantage is increased memory usage by your
	* application.
	* 
	* See Also:
	* <FlashrCache>
	**/
	var cacheQueries:Boolean = false;
	/**
	* Variable: cacheObeyOrder
	* Whether to ensure that cached results are returned in the same order as they are added to the queue
	* (i.e. whether to totally skip the queue when you have a cached result). It is normally better to skip
	* the queue and so get an instant result for cached calls (hence the default value of false) but in some
	* cases you want to ensure that the results are returned in the same order as you queue the queries...
	*/
	var cacheObeyOrder:Boolean = false;
	
	/**
	* Variable: cache
	* Variable holding the <FlashrCache> instance used to hold cached query results where <cacheQueries>
	* is true.
	**/
	function get cache():FlashrCache
	{
		return FlashrCache.getInstance();
	}
	function set cache(cache:FlashrCache):Void
	{
		// do nothing...
	}
	private var _cache:FlashrCache;
	/**
	* Variable: queue
	* The <FlashrRequestQueue>.
	**/
	function get queue():FlashrRequestQueue
	{
		return FlashrRequestQueue.getInstance();
	}
	function set queue(q:FlashrRequestQueue):Void
	{
		LogWrapper.getLog().error("Trying to set Flashr.queue - this does nothing so don't!");
	}
	
	/**
	* Variable: queuePriority
	* The priority to add requests to the queue at... The lower this number when you add a request to
	* the queue the quicker that request will be actioned.
	**/
	var queuePriority:Number;
	
	/**
	* Variable: queryMaxFrequency
	* The number of milliseconds to wait between subsequent queries to flickr.com (Flickr's TOS request no more
	* than one request per second)
	**/
	var queryMaxFrequency:Number = 1000;
	
	private var _queueUpdateInterval:Number;
	private var _lastQueryTimestamp:Number = 0;
	
	/**
	* Variable: history
	* The <FlashrRequestHistory> if <saveHistory> is true
	**/
	function get history():FlashrResponseHistory
	{
		return FlashrResponseHistory.getInstance();
	}
	function set history(frh:FlashrResponseHistory)
	{
		LogWrapper.getLog().error("Trying to set Flashr.history - this does nothing so don't!");
	}
	
	/**
	* Variable: saveHistory
	* Whether or not a <FlashrResponseHistory> should be saved of all the calls to flickr.com and
	* all the responses from it. Defaults to false so that memory isn't wasted unles you explicitly 
	* require this functionality.
	**/
	var saveHistory:Boolean = false;
	
	/**
	* Function: Flashr
	* 
	* Constructor -
	* as this class implements the Singleton pattern do not call directly
	* but rather access through <Flashr.getFlashr>.
	**/
	private function Flashr()
	{
		LogWrapper.getLog().debug(this+" created");
		awaitingResponse = false;
		_REST_ENDPOINT = "http://www.flickr.com/services/rest/";
		EventDispatcher.initialize(this);
	}

// flickr.auth methods

/**
* Function: authSetSecret
* 
* Use to set the value of the shared secret before any calls to methods like
* <authLogin> or <authCheckToken>
* 
* Parameters:
* secret		- Your applications shared secret.
* 
* See Also:
* <authLogin>
* 
* <authCheckToken>
* 
* <http://www.flickr.com/services/api/auth.spec.html>
* 
* <http://www.flickr.com/services/api/auth.howto.desktop.html>
* 
* <http://www.flickr.com/services/api/registered_keys.gne>
**/
public function authSetSecret(secret:String)
{
	_secret = secret;
}

/**
* Function: authLogin
* 
* Pops open a login window so that the user can authenticate with the
* flickr.com website. The user authenticates using the "Non-web based 
* app" method as described in section 9.2 of the auth spec.
* 
* The end effect of calling this function is that a window is opened
* with the relevant login / authenticate stuff on flickr.com. It is the
* responsibility of the user of this library to pop up a dialog box saying
* words to the effect of:
* 
* "This program requires your authentication before it can read/write/delete
* your photos on flickr.com. Please complete the authorisation process in the
* window which has popped up and then press the CONTINUE button below"
* 
* The CONTINUE button needs to call authGetToken which does the final stage
* of logging the user in.
* 
* Parameters:
* perms			-	A desired level of permission for actions 
* 				 	which the application wants to perform on behalf 
* 				 	of the user. "read", "write" or "delete".
* 
* See Also:
* <authGetToken>
* 
* <http://www.flickr.com/services/api/auth.spec.html>
* 
* <http://www.flickr.com/services/api/auth.howto.desktop.html>
* 
* <http://www.flickr.com/services/api/registered_keys.gne>
**/
public function authLogin(perms:String)
{
	authPerms = perms;
	authGetFrob(true);
}

/**
* Function: authGetFrob
* 
* Calls flickr.auth.getFrob to get a frob to be used during authentication. In most cases
* you would not call this method directly and would instead use <authLogin>.
* 
* Parameters:
* authIsLoggingIn	-	Set to true internally so that authLogin doesn't need to bother
* 						a <FlashrResponse>. If you call the function then leave this
* 						empty or put in the default (false).
* 
* See Also:
* <authLogin>
* 
* <FlashrResponse.onAuthGetFrob>
* 
* <http://flickr.com/services/api/flickr.auth.getFrob.html>
**/
public function authGetFrob(authIsLoggingIn:Boolean):FlashrRequest
{
	if (authIsLoggingIn) {
		_authIsLoggingIn = true;
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.auth.getFrob", null, null, true));
}

/**
* Function: authGetToken
* 
* Calls flickr.auth.getToken to get the auth token for the given frob, if one has 
* been attached.
* 
* Should be called after the user has pressed CONTINUE on the status window
* you opened when calling authLogin.
* 
* If a valid token is returned (e.g. if the frob was correct and valid) then the
* user is logged in for all further API calls from this session and
* <FlashrResponse.onAuthCheckToken> will be fired. If there is a problem
* then <FlashrResponse.onError> is fired.
* 
* See Also:
* <authLogin>
* 
* <FlashrResponse.onAuthCheckToken>
* 
* <http://www.flickr.com/services/api/flickr.auth.getToken.html>
* 
**/
public function authGetToken():FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.auth.getToken", {frob:_authFrob}, null, true));
}

/**
* Function: authCheckToken
* 
* Calls flickr.auth.checkToken to get the credentials attached to an 
* authentication token. You can use this to log in if you have auth'd
* an app in a previous session and saved the token e.g. in a SharedObject.
* If the token is still valid, the user will become logged in and 
* <FlashrResponse.onAuthCheckToken> will be fired. If the token is
* invalid then <FlashrResponse.onError> will be fired.
* 
* Parameters:
* authToken			- The authentication token to check.
* 
* See Also:
* <FlashrResponse.onAuthCheckToken>
* 
* <http://www.flickr.com/services/api/flickr.auth.checkToken.html>
**/
public function authCheckToken(authToken:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.auth.checkToken", {auth_token:authToken}, null, true));
}

/**
* Function: authLogout
* 
* Simple function which forgets the information that was making this user be
* logged in. The app will remain authenticated against flickr.com until the
* user visits <http://www.flickr.com/services/auth/list.gne> but the app will
* have to reapply for frob and then a token to use this authentication...
**/
public function authLogout()
{
	_authToken = "";
	authPerms = "none";
	delete _authFrob;
	delete authUser;
	//getURL("http://www.flickr.com/services/auth/list.gne", "_blank");
}

/**
* Function: _authOpenLogin
* 
* Internal function which generates the relevant link and pops open the
* login window on flickr.com. Triggered when a frob (necessary to generate
* the link) is returned from flickr.auth.getFrob.
* 
* See Also:
* <http://www.flickr.com/services/api/flickr.auth.getFrob.html>
* 
* <http://www.flickr.com/services/api/auth.spec.html>
* 
* <http://www.flickr.com/services/api/auth.howto.desktop.html>
* 
* <http://www.flickr.com/services/api/registered_keys.gne>
**/
private function _authOpenLogin()
{
	var args:Object = {api_key:_apiKey, perms:authPerms, frob:_authFrob};
	var loginUrl:String = _AUTH_ENDPOINT+"?";	
	for (var argumentName:String in args) {
		loginUrl += argumentName+"="+args[argumentName] + "&";
	}
	loginUrl += "api_sig="+_authSignCall(args);
	getURL(loginUrl, "_blank");
	/**
	* TODO - need to somehow display the "click once you have authorised" message here...
	* or is that just up to the application developer?
	* Maybe better to leave it up to the application developer as detailed in the
	* NaturalDocs to authLogin...
	**/
}


/**
* Function: _authSignCall
* 
* Internal, private function which is used to generate the api_sig used to
* sign a call.
* 
* Parameters:
* urlArguments		-	An Object containing name/value pairs which describe all of the
* 				  		arguments on the URL.
* 
* See Also:
* <http://www.flickr.com/services/api/auth.spec.html>
* 
* <http://www.flickr.com/services/api/auth.howto.desktop.html>
* 
* <http://www.flickr.com/services/api/registered_keys.gne>
**/
function _authSignCall(urlArguments:Object):String
{
	var toSort:Array = [];
	for (var argument:String in urlArguments) {
		toSort.push({name:argument, value:urlArguments[argument]});
	}
	toSort.sortOn("name");
	var signature:String = _secret;
	for (var i:Number=0; i<toSort.length; i++) {
		signature += toSort[i].name + toSort[i].value;
	}
	return MD5.calculate(signature);
}

// /flickr.auth methods

// flickr.contacts methods

/**
* Function: contactsGetList
* 
* Calls flickr.contacts.getList to get a list of contacts for the calling user.
* 
* Parameters:
* filter			-	An optional filter of the results. One of "friends", "friends", "both"
* 						or "neither"
* 
* See Also:
* <FlashrResponse.onContactsGetList>
* 
* <http://flickr.com/services/api/flickr.contacts.getList.html>
**/
public function contactsGetList(filter:String):FlashrRequest
{
	var params:Object = {};
	var additionalArgs:Object = {};
	if (filter) {
		params.filter = filter;
		additionalArgs.filter = filter;
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.contacts.getList", params, additionalArgs));
}
/**
* Function: contactsGetPublicList
* 
* Calls flickr.contacts.getPublicList to get a list of contacts a user.
* 
* Parameters:
* nsid 				- 	The nsid of the user whose contacts you want to get.
* 
* See Also:
* <FlashrResponse.onContactsGetPublicList>
* 
* <http://flickr.com/services/api/flickr.contacts.getPublicList.html>
**/
public function contactsGetPublicList(nsid:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.contacts.getPublicList", {user_id:nsid}, {nsid:nsid}));
}

// /flickr.contacts methods	

// flickr.favorites methods

/**
* Function: favoritesAdd
* 
* Calls flickr.favorites.add to add a photo to a user's favorites list.
* 
* Parameters:
* photoId 			- 	The id of the photo you want to add.
* 
* See Also:
* <FlashrResponse.onFavoritesAdd>
* 
* <http://www.flickr.com/services/api/flickr.favorites.add.html>
**/
public function favoritesAdd(photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.favorites.add", {photo_id:photoId}, {photoId:photoId}));
}
/**
* Function: favoritesGetList
* 
* Calls flickr.favorites.getList to get a list of the user's favorite photos.
* 
* Parameters:
* userId 			- 	The id of the user to get favourites for [Optional: default=logged in user]
* extras			-	A comma-delimited list of extra information to fetch for each returned record.
* 						Currently supported fields are:
* 						license, date_upload, date_taken, owner_name, icon_server, original_format.
* perPage 			- 	The number of results to get per page [Optional: default=100]
* page 				- 	The page of results to get [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onFavoritesGetList>
* 
* <http://www.flickr.com/services/api/flickr.favorites.getList.html>
**/
public function favoritesGetList(userId:String, extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {};
	if (userId) params.user_id = userId;
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	var additionalArguments:Object = {};
	// if no user id is passed we are trying to get the faves for the logged in user..
	additionalArguments.userId = userId ? userId : authUser.nsid;
	return queueRequest(FlashrRequest.generateRequest("flickr.favorites.getList", params, additionalArguments));
}
/**
* Function: favoritesGetPublicList
* 
* Calls flickr.favorites.getPublicList to get a list of favorite public photos for the given user.
* 
* Parameters:
* userId 			- 	The id of the user to get favourites for
* extras			-	A comma-delimited list of extra information to fetch for each returned record.
* 						Currently supported fields are:
* 						license, date_upload, date_taken, owner_name, icon_server.
* perPage 			- 	The number of results to get per page [Optional: default=100]
* page 				- 	The page of results to get [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onFavoritesGetPublicList>
* 
* <http://www.flickr.com/services/api/flickr.favorites.getPublicList.html>
**/
public function favoritesGetPublicList(userId:String, extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {user_id:userId};
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.favorites.getPublicList", params, {userId:userId}));
}
/**
* Function: favoritesRemove
* 
* Calls flickr.favorites.remove to remove a photo from a user's favorites list.
* 
* Parameters:
* photoId 				- 	The id of the photo you want to add.
* 
* See Also:
* <FlashrResponse.onFavoritesRemove>
* 
* <http://www.flickr.com/services/api/flickr.favorites.remove.html>
**/
public function favoritesRemove(photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.favorites.remove", {photo_id:photoId}, {photoId:photoId}));
}


// /flickr.favorites methods

// flickr.groups.pools methods
/**
* Function: groupsPoolsAdd
* Add a photo to a group's pool.
* 
* Parameters:
* photoId				-	The id of the photo to add to the group pool. The photo must belong to the 
* 							calling user.
* groupId				-	The NSID of the group who's pool the photo is to be added to.
* 
* See Also:
* <FlashrResponse.onGroupsPoolsAdd>
* 
* <http://www.flickr.com/services/api/flickr.groups.pools.add.html>
**/
function groupsPoolsAdd(photoId:String, groupId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.groups.pools.add", {photo_id:photoId, group_id:groupId}, {photoId:photoId, groupId:groupId}));
}
/**
* Function: groupsPoolsGetContext
* Gets next and previous photos for a photo in a group pool.
* 
* Parameters:
* photoId				-	The id of the photo to fetch the context for.
* groupId				-	The nsid of the group who's pool to fetch the photo's context for.
* 
* See Also:
* <FlashrResponse.onGroupsPoolsAdd>
* 
* <http://www.flickr.com/services/api/flickr.groups.pools.add.html>
**/
function groupsPoolsGetContext(photoId:String, groupId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.groups.pools.getContext", {photo_id:photoId, group_id:groupId}, {photoId:photoId, groupId:groupId}));
}
/**
* Function: groupsPoolsGetGroups
* Gets a list of groups to which you can add photos.
* 
* See Also:
* <FlashrResponse.onGroupsPoolsGetGroups>
* 
* <http://www.flickr.com/services/api/flickr.groups.pools.getGroups.html>
**/
function groupsPoolsGetGroups():FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.groups.pools.getGroups"));
}
/**
* Function: groupsPoolsGetPhotos
* 
* Gets a list of pool photos for a given group, based on the permissions of the group and the user 
* logged in (if any).
* 
* Parameters:
* groupId				-	The id of the group who's pool you which to get the photo list for.
* tags					-	A tag to filter the pool with. At the moment only one tag at a time is supported.
* extras				-	A comma-delimited list of extra information to fetch for each returned record.
* 							Currently supported fields are:
* 							license, date_upload, date_taken, owner_name, icon_server, original_format.
* perPage 				- 	The number of photos to return per page. [Optional: default=100]
* page 					- 	The page of results to return. [Optional: default=1]
* userId				-	The nsid of a user. Specifiying this parameter will retrieve for you only 
* 							those photos that the user has contributed to the group pool.
* 							[Optional]
* 
* See Also:
* <FlashrResponse.onGroupsPoolsGetPhotos>
* 
* <http://www.flickr.com/services/api/flickr.groups.pools.getPhotos.html>
**/
function groupsPoolsGetPhotos(groupId:String, tags:String, extras:String, perPage:Number, page:Number, userId:String):FlashrRequest
{
	var params:Object = {group_id:groupId};
	if (tags) {	params.tags = tags;	}
	if (extras) { params.extras = extras; }
	if (perPage) { params.per_page = perPage; }
	if (page) {	params.page = page;	}
	if (userId) { params.user_id = userId; }
	return queueRequest(FlashrRequest.generateRequest("flickr.groups.pools.getPhotos", params, {groupId:groupId}));
}
/**
* Function: groupsPoolsRemove
* Remove a photo from a group pool.
* 
* Parameters:
* photoId				-	The id of the photo to remove from the group pool. The photo must either be 
* 							owned by the calling user of the calling user must be an administrator of 
* 							the group
* groupId				-	The NSID of the group who's pool the photo is to removed from.
**/
function groupsPoolsRemove(photoId:String, groupId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.groups.pools.remove", {photo_id:photoId, group_id:groupId}, {photoId:photoId, groupId:groupId}));
}
// /flickr.groups.pools methods

// flickr.intrestingness methods

/**
* Function: interestingnessGetList
* Get the list of interesting photos for the most recent day or a user-specified date
* 
* Parameters:
* date					-	A specific date, to return interesting photos for (or undefined for the most
* 							recent days photos)
* extras				-	A comma-delimited list of extra information to fetch for each returned record.
* 							Currently supported fields are:
* 							license, date_upload, date_taken, owner_name, icon_server, original_format.
* perPage 				- 	The number of photos to return per page. [Optional: default=100]
* page 					- 	The page of results to return. [Optional: default=1]
**/
function interestingnessGetList(date:Date, extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {};
	if (date) params.date = DateFormat.dateToIso(date, true);
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.interestingness.getList", params, {date:date}));
}

// /flickr.intrestingness methods

// flickr.people methods

/**
* Function: peopleFindByEmail
* 
* Calls flickr.people.findByEmail to get a user's NSID, given their email address.
* 
* Parameters:
* email 		- 	The email address of the person whose nsid you want
* 
* See Also:
* <FlashrResponse.onPeopleFindByEmail>
* 
* <http://flickr.com/services/api/flickr.people.findByEmail.html>
**/
function peopleFindByEmail(email:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.people.findByEmail", {find_email:email}, {email:email}));
}
/**
* Function: peopleFindByUsername
* 
* Calls flickr.people.findByUsername to get a user's NSID, given their username.
* 
* Parameters:
* username 		- 	The username of the person whose nsid you want
* 
* See Also:
* <FlashrResponse.onPeopleFindByUsername>
* 
* <http://flickr.com/services/api/flickr.people.findByUsername.html>
**/
function peopleFindByUsername(username:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.people.findByUsername", {username:username}));
}
/**
* Function: peopleGetInfo
* 
* Calls flickr.people.getInfo to get information about a user.
* 
* Parameters:
* nsid 			- 	The nsid of the user you want information about.
* 
* See Also:
* <FlashrResponse.onPeopleGetInfo>
* 
* <http://flickr.com/services/api/flickr.people.getInfo.html>
**/
function peopleGetInfo(nsid:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.people.getInfo", {user_id:nsid}));
}
/**
* Function: peopleGetPublicGroups
* 
* Calls flickr.people.getPublicGroups to get the list of public groups a user is a member of.
* 
* Parameters:
* nsid 			- 	The nsid of the user whose groups you want to know about.
* 
* See Also:
* <FlashrResponse.onPeopleGetPublicGroups>
* 
* <http://www.flickr.com/services/api/flickr.people.getPublicGroups.html>
**/
function peopleGetPublicGroups(nsid:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.people.getPublicGroups", {user_id:nsid}, {nsid:nsid}));
}
/**
* Function: peopleGetPublicPhotos
* 
* Calls flickr.people.getPublicPhotos to get a list of public photos for the given user.
* 
* Parameters:
* nsid 			- 	The nsid of the user whose photos you want to know about.
* extras		-	A comma-delimited list of extra information to fetch for each returned record.
* 					Currently supported fields are:
* 					license, date_upload, date_taken, owner_name, icon_server, original_format.
* perPage 		- 	The number of photos to return per page. [Optional: default=100]
* page 			- 	The page of results to return. [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onPeopleGetPublicPhotos>
* 
* <http://flickr.com/services/api/flickr.people.getPublicPhotos.html>
**/
function peopleGetPublicPhotos(nsid:String, extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {user_id:nsid};
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.people.getPublicPhotos", params, {nsid:nsid}));
}
/**
* Function: peopleGetUploadStatus
* 
* Calls flickr.people.getUploadStatus to get information for the calling user related to photo uploads.
* 
* See Also:
* <FlashrResponse.onPeopleGetUploadStatus>
* 
* <http://flickr.com/services/api/flickr.people.getUploadStatus.html>
**/
function peopleGetUploadStatus():FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.people.getUploadStatus"));
}

// /flickr.people methods

// flickr.photos methods

/**
* Function: photosAddTags
* 
* Calls flickr.photos.addTags to add tags to a given photo.
* 
* Parameters:
* photoId		-	The id of the photo to add tags to.
* tags			- 	The tags to add to the photo.
* 
* See Also:
* <FlashrResponse.onPhotosAddTags>
* 
* <http://flickr.com/services/api/flickr.photos.addTags.html>
**/
function photosAddTags (photoId:String, tags:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.addTags", {photo_id:photoId, tags:tags}, {tags:tags, photoId:photoId}));
}
/**
* Function: photosGetAllContexts
* 
* Returns all visble sets and pools the photo belongs to.
* 
* Parameters:
* photoId		-	The photo to return information for.
* 
* See Also:
* <FlashrResponse.onPhotosGetAllContexts>
* 
* <http://www.flickr.com/services/api/flickr.photos.getAllContexts.html>
**/
function photosGetAllContexts(photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getAllContexts", {photo_id:photoId}, {photoId:photoId}));
}
/**
* Function: photosGetContactsPhotos
* 
* Calls flickr.photos.getContactsPhotos to fetch a list of recent photos 
* from the calling users' contacts.
* 
* Parameters:
* count			- 	Number of photos to return. Defaults to 10, maximum 50. 
* 				  	This is only used if single_photo is not passed.
* justFriends	- 	set as 1 to only show photos from friends and family (excluding 
* 				  	regular contacts). [Optional: default=0].
* singlePhoto	- 	Only fetch one photo (the latest) per contact, instead of 
* 				  	all photos in chronological order [Optional: default=0].
* includeSelf	- 	Set to 1 to include photos from the calling user. 
* 				  	[Optional: default=0].
* extras		-	A comma-delimited list of extra information to fetch for each returned 
* 					record. Currently supported fields are: license, date_upload, date_taken, 
* 					owner_name, icon_server, original_format.
* 
* See Also:
* <FlashrResponse.onPhotosGetContactsPhotos>
* 
* <http://flickr.com/services/api/flickr.photos.getContactsPhotos.html>
**/
function photosGetContactsPhotos (count:Number, justFriends:Number, singlePhoto:Number, includeSelf:Number, extras:String):FlashrRequest
{
	var params:Object = {};
	if (count != undefined) params.count = count;
	params.just_friends = justFriends ? 1 : 0;
	params.single_photo = singlePhoto ? 1 : 0;
	params.include_self = includeSelf ? 1 : 0;
	if (extras) params.extras = extras;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getContactsPhotos", params));
}
/**
* Function: photosGetContactsPublicPhotos
* 
* Calls flickr.photos.getContactsPublicPhotos to fetch a list of recent public 
* photos from a users' contacts.
* 
* Parameters:
* nsid			- 	The NSID of the user to fetch photos for.
* count			- 	Number of photos to return. Defaults to 10, maximum 50. 
* 				  	This is only used if single_photo is not passed. [Optional: default=10].
* justFriends	- 	set as 1 to only show photos from friends and family (excluding 
* 				  	regular contacts). [Optional: default=0].
* singlePhoto	- 	Only fetch one photo (the latest) per contact, instead of 
* 				  	all photos in chronological order [Optional: default=0].
* includeSelf	- 	Set to 1 to include photos from the calling user. 
* 				  	[Optional: default=0].
* extras		-	A comma-delimited list of extra information to fetch for each returned 
* 					record. Currently supported fields are: license, date_upload, date_taken, 
* 					owner_name, icon_server, original_format.
* 
* See Also:
* <FlashrResponse.onPhotosGetContactsPublicPhotos>
* 
* <http://www.flickr.com/services/api/flickr.photos.getContactsPublicPhotos.html>
**/
function photosGetContactsPublicPhotos (nsid:String, count:Number, justFriends:Number, singlePhoto:Number, includeSelf:Number, extras:String):FlashrRequest
{
	var params:Object = {user_id:nsid};
	if (count != undefined) params.count = count;
	params.just_friends = justFriends ? 1 : 0;
	params.single_photo = singlePhoto ? 1 : 0;
	params.include_self = includeSelf ? 1 : 0;
	if (extras) {
		params.extras = extras;
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getContactsPublicPhotos", params, {nsid:nsid}));
}
/**
* Function: photosGetContext
* 
* Calls flickr.photos.getContext to return next and previous photos for a 
* photo in a photostream.
* 
* Parameters:
* photoId		- 	The id of the photo to fetch the context for.
* 
* See Also:
* <FlashrResponse.onPhotosGetContext>
* 
* <http://www.flickr.com/services/api/flickr.photos.getContext.html>
**/
function photosGetContext (photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getContext", {photo_id:photoId}, {photoId:photoId}));
}
/**
* Function: photosGetCounts
* 
* Calls flickr.photos.getCounts to get a list of photo counts for the 
* given date ranges for the calling user.
* 
* Parameters:
* dates			-	A comma delimited list of unix timestamps
* 					denoting the periods to return counts for. 
* 					They should be specified smallest first.
* 					[Optional: if not present then takenDates used instead.
* takenDates	-	A comma delimited list of mysql datetimes
* 					denoting the periods to return counts for. 
* 					They should be specified smallest first.
* 					[Optional: if not present then dates used instead.
* 
* Note:
* * The dates parameter is passed as UNIX Timestamps while the takenDates parameter
* is passed as MySQL Timestamps (e.g. 2005-03-21).
* * If you want to pass takenDates then pass "" (empty string) or undefined as the value
* of dates.
* 
* TODO:
* Should we change this method to accept Arrays of Date Objects rather than comma delimited
* Strings? Then we could sort it making order unimportant and could abstract away the fact
* that Flickr stores dates uploaded and dates taken in different formats.
* Maybe it would be better if the method's arguments were dates (an Array of Date 
* Objects) and type (Flashr.DATES_UPLOADED or Flashr.DATES_TAKEN)?
* 
* See Also:
* <FlashrResponse.onPhotosGetCounts>
* 
* <http://www.flickr.com/services/api/flickr.photos.getCounts.html>
**/
function photosGetCounts (dates:String, takenDates:String):FlashrRequest
{
	var params:Object = {};
	var additionalArgs:Object = {};
	if (dates != undefined && dates != "") {
		params.dates = dates;
		additionalArgs.dateType = "uploaded";
	}
	if (takenDates != undefined) {
		params.taken_dates = takenDates;
		additionalArgs.dateType = "taken";
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getCounts", params, additionalArgs));
}
/**
* Function: photosGetExif
* 
* Calls flickr.photos.getExif to return a list of EXIF/TIFF/GPS tags for a given photo.
* 
* Parameters:
* photoId		- 	The id of the photo to fetch the context for.
* secret		- 	The secret for the photo. If the correct secret is passed then permissions
* 					checking is skipped. This enables the 'sharing' of individual photos by 
* 					passing around the id and secret.
* 
* See Also:
* <FlashrResponse.onPhotosGetExif>
* 
* <ExifData>
* 
* <http://www.flickr.com/services/api/flickr.photos.getExif.html>
**/
function photosGetExif (photoId:String, secret:String):FlashrRequest
{
	var params:Object = {photo_id:photoId};
	if (secret) params.secret = secret;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getExif", params));
}
/**
* Function: photosGetInfo
* 
* Calls flickr.photos.getInfo to get information about a photo..
* 
* Parameters:
* photoId		- 	The id of the photo to fetch the context for.
* secret		- 	The secret for the photo. If the correct secret is passed then permissions
* 					checking is skipped. This enables the 'sharing' of individual photos by 
* 					passing around the id and secret.
* 
* See Also:
* <FlashrResponse.onPhotosGetInfo>
* 
* <http://www.flickr.com/services/api/flickr.photos.getInfo.html>
**/
function photosGetInfo (photoId:String, secret:String):FlashrRequest
{
	var params:Object = {photo_id:photoId};
	if (secret) params.secret = secret;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getInfo", params));
}
/**
* Function: photosGetNotInSet
* 
* Calls flickr.photos.getNotInSet to get a list of your photos that are not part of any sets.
* 
* Parameters:
* extras		-	A comma-delimited list of extra information to fetch for each returned record.
* 					Currently supported fields are:
* 					license, date_upload, date_taken, owner_name, icon_server, original_format
* perPage 		- 	The number of photos to return per page. [Optional: default=100]
* page 			- 	The page of results to return. [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onPhotosGetNotInSet>
* 
* <http://www.flickr.com/services/api/flickr.photos.getNotInSet.html>
**/
function photosGetNotInSet (extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {};
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getNotInSet", params));
}
/**
* Function: photosGetPerms
* 
* Calls flickr.photos.getPerms to get permissions for a photo.
* 
* Parameters:
* photoId 		- 	The id of the photo whose permissions you want
* 
* See Also:
* <FlashrResponse.onPhotosGetNotInSet>
* 
* <http://www.flickr.com/services/api/flickr.photos.getPerms.html>
**/
function photosGetPerms (photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getPerms", {photo_id:photoId}));
}
/**
* Function: photosGetRecent
* 
* Calls flickr.photos.getRecent to get a list of the latest public photos uploaded to flickr.
* 
* Parameters:
* extras		-	A comma-delimited list of extra information to fetch for each returned record.
* 					Currently supported fields are:
* 					license, date_upload, date_taken, owner_name, icon_server, original_format
* perPage 		- 	The number of photos to return per page. [Optional: default=100]
* page 			- 	The page of results to return. [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onPhotosGetRecent>
* 
* <http://www.flickr.com/services/api/flickr.photos.getRecent.html>
**/
function photosGetRecent (extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {};
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getRecent", params));
}
/**
* Function: photosGetSizes
* 
* Calls flickr.photos.getRecent to get the available sizes for a photo.
* 
* Parameters:
* photoId	 		- 	The id of the photo to fetch size information for.
* 
* See Also:
* <FlashrResponse.onPhotosGetSizes>
* 
* <http://www.flickr.com/services/api/flickr.photos.getSizes.html>
**/
function photosGetSizes (photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getSizes", {photo_id:photoId}, {photoId:photoId}));
}
/**
* Function: photosGetUntagged
* 
* Calls flickr.photos.getUntagged to get a list of your photos with no tags.
* 
* Parameters:
* extras		-	A comma-delimited list of extra information to fetch for each returned record.
* 					Currently supported fields are:
* 					license, date_upload, date_taken, owner_name, icon_server
* perPage 		- 	The number of photos to return per page. [Optional: default=100]
* page 			- 	The page of results to return. [Optional: default=1]
* 
* See Also:
* <FlashrResponse.onPhotosGetUntagged>
* 
* <http://www.flickr.com/services/api/flickr.photos.getUntagged.html>
**/
function photosGetUntagged (extras:String, perPage:Number, page:Number):FlashrRequest
{
	var params:Object = {};
	if (extras) params.extras = extras;
	if (perPage) params.per_page = perPage;
	if (page) params.page = page;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.getUntagged", params));
}
/**
* Function: photosRemoveTag
* 
* Calls flickr.photos.removeTag to remove a tag from a photo.
* 
* Parameters:
* tagId	 		- 	The tag to remove from the photo. 
* 					This parameter should contain a tag id, as returned by 
* 					<photosGetInfo>.
* 
* See Also:
* <FlashrResponse.onPhotosRemoveTag>
* 
* <http://www.flickr.com/services/api/flickr.photos.removeTag.html>
**/
function photosRemoveTag (tagId:Number):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.removeTag", {tag_id:tagId}, {tagId:tagId}));
}
/**
* Function: photosSearch
* 
* Calls flickr.photos.search to get a list of photos matching some criteria.
* 
* You pass an object with one or many of the following attributes...
* 
* Attributes:
* user_id			-	The NSID of the user who's photo to search. If this parameter 
* 						isn't passed then everybody's public photos will be searched.
* tags				-	A comma-delimited list of tags. Photos with one or more of the 
* 						tags listed will be returned.
* tag_mode			-	Either 'any' for an OR combination of tags, or 'all' for an AND 
* 						combination. Defaults to 'any' if not specified.
* text				-	A free text search. Photos who's title, description or tags 
* 						contain the text will be returned.
* min_upload_date	-	Minimum upload date. Photos with an upload date greater than or 
* 						equal to this value will be returned. The date should be in the 
* 						form of a UNIX timestamp.
* max_upload_date	-	Maximum upload date. Photos with an upload date less than or 
* 						equal to this value will be returned. The date should be in the form 
* 						of a UNIX timestamp.
* min_taken_date	-	Minimum taken date. Photos with an taken date greater than or equal 
* 						to this value will be returned. The date should be in the form of a 
* 						MySQL datetime.
* max_taken_date	-	Maximum taken date. Photos with an taken date less than or equal to 
* 						this value will be returned. The date should be in the form of a MySQL 
* 						datetime.
* license			-	The license id for photos.
* extras			-	A comma-delimited list of extra information to fetch for each 
* 						returned record. Currently supported fields are:
* 						license, date_upload, date_taken, owner_name, icon_server, original_format
* per_page			-	Number of photos to return per page. If this argument is ommited, it 
* 						defaults to 100. The maximum allowed value is 500.
* page				-	The page of results to return. If this argument is ommited, it defaults to 1.
* sort				-	The order in which to sort returned photos. Deafults to date-posted-desc. 
* 						The possible values are: date-posted-asc, date-posted-desc, date-taken-asc 
* 						, date-taken-desc, interestingness-desc, interestingness-asc and relevance
* 
* TODO:
* Should probably change it to accept dates as Date Objects... Then you wouldn't have to worry about
* what type of date flickr expects for the different arguments.
* 
* See Also:
* <FlashrResponse.onPhotosSearch>
* 
* <http://www.flickr.com/services/api/flickr.photos.search.html>
**/
function photosSearch (params):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.search", params));
}
/**
* Function: photosSetDates
* 
* Calls flickr.photos.setDates to set one or both of the dates for a photo..
* 
* All parameters are optional but you must provide either dateTaken or datePosted (or
* both if you like).
* 
* Dates are passed as Flash Date Objects so you don't need to worry about the fact that
* flickr uses two different formats to represent it's dates...
* 
* Parameters:
* photoId 		- 	The id of the photo to edit dates for.
* datePosted	- 	The Date the photo was uploaded to flickr.
* dateTaken		-	The Date the photo was taken.
* dateTakenGran	-	The granularity of the date the photo was taken.
* 
* See Also:
* <FlashrResponse.onPhotosSetDates>
* 
* <http://flickr.com/services/api/flickr.photos.setDates.html>
* 
* <http://flickr.com/services/api/misc.dates.html>
**/
function photosSetDates (photoId:String, datePosted:Date, dateTaken:Date, dateTakenGran:Number):FlashrRequest
{
	var additionalArgs:Object;
	additionalArgs.photoId = photoId; 
	
	var params:Object = {photo_id:photoId};
	if (datePosted instanceof Date) {
		params.date_posted = datePosted.getTime()/1000;
		if (!awaitingResponse) additionalArgs.datePosted = params.date_posted;
	}
	if (dateTaken instanceof Date) {
		params.date_taken = DateFormat.dateToIso(dateTaken);
		if (!awaitingResponse) additionalArgs.dateTaken = params.date_taken;
	}
	if (dateTakenGran != undefined) params.date_taken_granularity = dateTakenGran;
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.setDates", params, additionalArgs));
}
/**
* Function: photosSetMeta
* 
* Calls flickr.photos.setMeta to set the meta information for a photo.
* 
* Parameters:
* photoId 		- 	The id of the photo to edit dates for.
* title			- 	The title for the photo.
* description	-	The description for the photo.
* 
* See Also:
* <FlashrResponse.onPhotosSetMeta>
* 
* <http://flickr.com/services/api/flickr.photos.setMeta.html>
**/
function photosSetMeta (photoId:String, title:String, description:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.setMeta", {photo_id:photoId, title:title, description:description}, {photoId:photoId, title:title, description:description}));
}
/**
* Function: photosSetPerms
* 
* Calls flickr.photos.setPerms to set permissions for a photo..
* 
* Parameters:
* photoId 		- 	The id of the photo to set permissions for.
* isPublic		- 	1 to set the photo to public, 0 to set it to private.
* isFriend		-	1 to make the photo visible to friends when private, 0 to not.
* isFamily		-	1 to make the photo visible to family when private, 0 to not.
* permComment	-	who can add comments to the photo and it's notes. See Flashr.PERM_* for options.
* permAddMeta	-	who can add notes and tags to the photo. See Flashr.PERM_* for options.
* 
* See Also:
* <Flashr.PERM_NOBODY>
* 
* <Flashr.PERM_FRIENDS>
* 
* <Flashr.PERM_CONTACTS>
* 
* <Flashr.PERM_EVERYBODY>
* 
* <FlashrResponse.onPhotosSetMeta>
* 
* <http://flickr.com/services/api/flickr.photos.setMeta.html>
**/
function photosSetPerms (photoId:String, isPublic:Number, isFriend:Number, isFamily:Number, permComment:Number, permAddMeta:Number):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.setPerms", {photo_id:photoId, is_public:isPublic, is_friend:isFriend, is_family:isFamily, perm_comment:permComment, perm_addmeta:permAddMeta}, {photoId:photoId}));
}
/**
* Function: photosSetTags
* 
* Calls flickr.photos.setTags to set the tags for a photo.
* 
* Parameters:
* photoId 		- 	The id of the photo to edit dates for.
* tags			- 	All tags for the photo (as a single space-delimited string).
* 
* See Also:
* <FlashrResponse.onPhotosSetTags>
* 
* <Photo.setTags> - 
* There are some problems listed here...
* 
* <http://flickr.com/services/api/flickr.photos.setTags.html>
**/
function photosSetTags (photoId:String, tags:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.setTags", {photo_id:photoId, tags:tags}, {photoId:photoId, tags:tags}));
}

// /flickr.photos methods

// flickr.photos.comments methods

/**
* Function: photosCommentsGetList
* 
* Calls flickr.photos.comments.getList to get the comments for a photo.
* 
* Parameters:
* photoId 		- 	The id of the photo to get comments for.
* 
* See Also:
* <FlashrResponse.onPhotosCommentsGetList>
* 
* <Photo.getComments>
* 
* <http://flickr.com/services/api/flickr.photos.comments.getList.html>
**/
function photosCommentsGetList (photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photos.comments.getList", {photo_id:photoId}));
}

// /flickr.photos.comments methods

// flickr.photosets methods
/**
* Function: photosetsAddPhoto
* 
* Add a photo to the end of an existing photoset.
* 
* Parameters:
* photosetId		-	The id of the photoset to add a photo to.
* photoId			-	The id of the photo to add to the set.
* 
* See Also:
* <FlashrResponse.onPhotosetsAddPhoto>
* 
* <http://www.flickr.com/services/api/flickr.photosets.addPhoto.html>
**/
function photosetsAddPhoto(photosetId:String, photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.addPhoto", {photoset_id:photosetId, photo_id:photoId}, {photosetId:photosetId, photoId:photoId}));
}
/**
* Function: photosetsCreate
* 
* Create a new photoset for the calling user.
* 
* Parameters:
* title				-	A title for the photoset.
* description		-	A description of the photoset. May contain limited html.
* primaryPhotoId	-	The id of the <Photo> to represent this set. The photo must belong to the calling user.
* 
* See Also:
* <FlashrResponse.onPhotosetsCreate>
* 
* <http://www.flickr.com/services/api/flickr.photosets.create.html>
**/
function photosetsCreate(title:String, description:String, primaryPhotoId:Number):FlashrRequest 
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.create", {title:title, description:description, primary_photo_id:primaryPhotoId}, {title:title, description:description, primaryPhotoId:primaryPhotoId}));
}
/**
* Function: photosetsDelete
* 
* Delete a photoset.
* 
* Parameters:
* photosetId		-	The id of the <Photoset> to delete. It must be owned by the calling user.
* 
* See Also:
* <FlashrResponse.onPhotosetsDelete>
* 
* <http://www.flickr.com/services/api/flickr.photosets.delete.html>
**/
function photosetsDelete(photosetId:String):FlashrRequest 
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.delete", {photoset_id:photosetId}, {photosetId:photosetId}));
}
/**
* Function: photosetsEditMeta
* 
* Modify the meta-data for a photoset.
* 
* Parameters:
* photosetId			-	The id of the <Photoset> to modify.
* title					-	The new title for the photoset.
* description			-	A description of the photoset. May contain limited html.
* 
* See Also:
* <FlashrResponse.onPhotosetsEditMeta>
* 
* <http://www.flickr.com/services/api/flickr.photosets.editMeta.html>
**/
function photosetsEditMeta(photosetId:String, title:String, description:String):FlashrRequest
{
	var params:Object = {photoset_id:photosetId, title:title};
	if (description) {
		params.description = description;
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.editMeta", params, {photosetId:photosetId, title:title, description:description}));
}
/**
* Function: photosetsEditPhotos
* 
* Modify the photos in a photoset. Use this method to add, remove and re-order photos.
* 
* Parameters:
* photosetId			-	The id of the photoset to modify. The photoset must belong to the calling user.
* primaryPhotoId		-	The id of the photo to use as the 'primary' photo for the set. This id must 
* 							also be passed along in photo_ids list argument.
* photoIds				-	A comma-delimited list of photo ids to include in the set. They will appear in 
* 							the set in the order sent. This list must contain the primary photo id. All photos 
* 							must belong to the owner of the set. This list of photos replaces the existing list. 
* 							Call <photosetsAddPhoto> to append a photo to a set.
* 
* See Also:
* <FlashrResponse.onPhotosetsEditPhotos>
* 
* <http://www.flickr.com/services/api/flickr.photosets.editPhotos.html>
**/
function photosetsEditPhotos(photosetId:String, primaryPhotoId:Number, photoIds:Array):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.editPhotos", {photoset_id:photosetId, primary_photo_id:primaryPhotoId, photo_ids:photoIds.join(",")}, {photosetId:photosetId, primaryPhotoId:primaryPhotoId, photoIds:photoIds}));
}
/**
* Function: photosetsGetContext
* 
* Returns next and previous photos for a photo in a set.
* 
* Parameters:
* photoId				-	The id of the <Photo> to fetch the context for.
* photosetId			-	The id of the <Photoset> for which to fetch the <Photo>'s context.
* 
* See Also:
* <FlashrResponse.onPhotosetsGetContext>
* 
* <http://www.flickr.com/services/api/flickr.photosets.getContext.html>
**/
function photosetsGetContext(photoId:String, photosetId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.getContext",  {photo_id:photoId, photoset_id:photosetId},  {photoId:photoId, photosetId:photosetId}));
}
/**
* Function: photosetsGetInfo
* 
* Gets information about a photoset.
* 
* Parameters:
* photosetId			-	The id of the <Photoset> to fetch information for.
* 
* See Also:
* <FlashrResponse.onPhotosetsGetInfo>
* 
* <http://www.flickr.com/services/api/flickr.photosets.getInfo.html>
**/
function photosetsGetInfo(photosetId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.getInfo", {photoset_id:photosetId}));
}
/**
* Function: photosetsGetList
* 
* Gets the photosets belonging to the specified user.
* 
* Parameters:
* userId				-	The nsid of the <Person> to get a photoset list for. If 
* 							none is specified, the calling user is assumed.
* 
* See Also:
* <FlashrResponse.onPhotosetsGetList>
* 
* <http://www.flickr.com/services/api/flickr.photosets.getList.html>
**/
function photosetsGetList(userId:String):FlashrRequest
{
	var params:Object = {};
	if (userId) params.user_id = userId;
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.getList", params, {userId:userId}));
}
/**
* Function: photosetsGetPhotos
* 
* Get the list of photos in a set.
* 
* Parameters:
* photosetId			-	The id of the <Photoset> to return the photos for.
* extras				-	A comma-delimited list of extra information to fetch for each returned record. 
* 							Currently supported fields are: license, date_upload, date_taken, owner_name, 
* 							icon_server, original_format.
* 
* See Also:
* <FlashrResponse.onPhotosetsGetPhotos>
* 
* <http://www.flickr.com/services/api/flickr.photosets.getPhotos.html>
**/
function photosetsGetPhotos(photosetId:String, extras:String):FlashrRequest
{
	var params:Object = {photoset_id:photosetId};
	if (extras) params.extras = extras;
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.getPhotos", params));
}
/**
* Function: photosetsOrderPhotosets
* 
* Set the order of photosets for the calling user.
* 
* Parameters:
* photosetIds			-	An array of photoset IDs, ordered with the set to show first, first in the list. 
* 							Any set IDs not given in the list will be set to appear at the end of the list, 
* 							ordered by their IDs.
* 
* See Also:
* <FlashrResponse.onPhotosetsOrderPhotosets>
* 
* <http://www.flickr.com/services/api/flickr.photosets.orderSets.html>
**/
function photosetsOrderPhotosets(photosetIds:Array):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.orderSets", {photoset_ids:photosetIds.join(",")}, {photosetIds:photosetIds}));
}
/**
* Function: photosetsRemovePhoto
* 
* Remove a photo from a photoset.
* 
* Parameters:
* photosetId			-	The id of the <Photoset> to remove a <Photo> from.
* photoId				-	The id of the <Photo> to remove from the set.
* 
* See Also:
* <FlashrResponse.onPhotosetsRemovePhoto>
* 
* <http://www.flickr.com/services/api/flickr.photosets.removePhoto.html>
**/
function photosetsRemovePhoto(photosetId:String, photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.photosets.removePhoto", {photoset_id:photosetId, photo_id:photoId}, {photosetId:photosetId, photoId:photoId}));
}
// /flickr.photosets methods

// flickr.tags methods

/**
* Function: tagsGetListPhoto
* 
* Calls flickr.tags.getListPhoto to get the tag list for a given photo.
* 
* Parameters:
* photoId		-	The id of the photo to return tags for.
* 
* See Also:
* <FlashrResponse.onTagsGetListPhoto>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.tags.getListPhoto.html>
**/
public function tagsGetListPhoto(photoId:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.tags.getListPhoto", {photo_id:photoId}));
}
/**
* Function: tagsGetListUser
* 
* Calls flickr.tags.getListUser to 
* get the tag list for a given user (or the currently logged in user).
* 
* Parameters:
* userId		-	The NSID of the user to fetch the tag list for. 
* 					[Optional: default=the currently logged in user (if any)]
* 
* See Also:
* <FlashrResponse.onTagsGetListUser>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.tags.getListUser.html>
**/
public function tagsGetListUser(userId:String):FlashrRequest
{
	var params:Object = {};
	if (userId)  params.user_id = userId;
	return queueRequest(FlashrRequest.generateRequest("flickr.tags.getListUser", params));
}
/**
* Function: tagsGetListUserPopular
* 
* Calls flickr.tags.getListUserPopular to
* get the popular tags for a given user (or the currently logged in user).
* 
* Parameters:
* userID 		-	The NSID of the user to fetch the tag list for. 
* 					[Optional: default=the currently logged in user (if any)]
* count 		-	Number of popular tags to return. [Optional: default=10]
* 
* See Also:
* <FlashrResponse.onTagsGetListUserPopular>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.tags.getListUserPopular.html>
**/
public function tagsGetListUserPopular(userId:String, count:Number):FlashrRequest
{
	var params:Object = {};
	if (userId)  params.user_id = userId;
	if (count) params.count = count;
	return queueRequest(FlashrRequest.generateRequest("flickr.tags.getListUserPopular", params));
}
/**
* Function: tagsGetRelated
* 
* Calls flickr.tags.getRelated to get a list of tags 'related' to the given tag, 
* based on clustered usage analysis.
* 
* Parameters:
* tag - The tag to fetch related tags for.
* 
* See Also:
* <FlashrResponse.onTagsGetRelated>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.tags.getRelated.html>
**/
public function tagsGetRelated(tag:String):FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.tags.getRelated", {tag:tag}));
}

// /flickr.tags methods


	
// flickr.test methods

/**
* Function: testEcho
* 
* Calls flickr.test.echo -
* forwards any arguments on to the Flickr API
* 
* Parameters:
* 1...n - Any number of parameters to echo back from the server [Optional]
* 
* See Also:
* <FlashrResponse.onTestEcho>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.test.echo.html>
**/
public function testEcho():FlashrRequest
{
	var params:Object = {};
	for (var i=0; i<arguments.length; i++) {
		params["argument"+(i+1)] = arguments[i];
	}
	return queueRequest(FlashrRequest.generateRequest("flickr.test.echo", params));
}
/**
* Function: testLogin
* 
* Calls flickr.test.login -
* A testing method which checks if the caller is logged in then returns their username.
* 
* See Also:
* <FlashrResponse.onTestLogin>
* 
* <FlashrResponse.onError>
* 
* <http://www.flickr.com/services/api/flickr.test.login.html>
**/
public function testLogin():FlashrRequest
{
	return queueRequest(FlashrRequest.generateRequest("flickr.test.login"));
}

// /flickr.test methods
	
	/**
	* Function: queueRequest
	* Queues a request for execution on flickr.com
	* // TODO: Add priority...
	* // TODO: Assign a unique ID, add to queue and return ID (for use later with setPriority etc)
	* 
	* Parameters:
	* request				-	The <FlashrRequest> to queue.
	* 
	* Returns:
	* The <FlashrRequest> that was queued (you can use this object to change the priority of a given request
	* etc)
	**/
	function queueRequest(request:FlashrRequest):FlashrRequest
	{
		if (cacheQueries && !cacheObeyOrder) {
			var responseEvent:FlashrResponseEvent = cache.getCached(request);
			if (responseEvent != undefined) {
				// dispatch cached version and skip the rest of the function...
				LogWrapper.getLog().debug("queueRequest - dispatching cached response for "+request);
				dispatchEvent(responseEvent);
				if( saveHistory) {
					history.addResponse(responseEvent);
				}
				return request;
			}
		}
		
		FlashrRequestQueue.getInstance().addToQueue(request, queuePriority);
		_executeNextRequest();
		return request;
	}
	
	private function _executeNextRequest()
	{
		if (!awaitingResponse) {
			if (getTimer() - _lastQueryTimestamp > queryMaxFrequency) {
				var request:FlashrRequest = FlashrRequestQueue.getInstance().getNext();
				if (!request) {
					clearInterval(_queueUpdateInterval);
					LogWrapper.getLog().info("Flashr queue empty");
					delete _queueUpdateInterval;
				} else {
					callMethod(request);
				}
			}
		}
		if (_queueUpdateInterval == undefined && !queue.isEmpty()) {
			_queueUpdateInterval = setInterval(this, "_executeNextRequest", 100);
		}
	}
		
	/**
	* Function: callMethod
	* 
	* Used to call a method on the flickr API.
	* 
	* Use <queueRequest> if you want to call a method on flickr.com which isn't implemented in Flashr
	* 
	* Parameters:
	* request				-	The <FlashrRequest> to call
	* 
	* See also:
	* <queueRequest>
	* 
	* <onAPIResponse>
	**/
	private function callMethod(request:FlashrRequest):Void
	{
		if (cacheQueries) {
			var responseEvent:FlashrResponseEvent = cache.getCached(request);
			if (responseEvent != undefined) {
				// dispatch cached version and skip the rest of the function...
				LogWrapper.getLog().debug("callMethod - dispatching cached response for "+request);
				dispatchEvent(responseEvent);
				if (saveHistory) {
					history.addResponse(responseEvent);
				}
				_executeNextRequest();
				return;
			}
		}
		if (awaitingResponse) {
			dispatchEvent({type:"onAPIResponse", target:this, status:STATUS_BUSY, errorMessage:STATUS_BUSY_DESCRIPTION});
		} else {
			_currentRequest = request;
			awaitingResponse = true;
			var url:String = _REST_ENDPOINT+_currentRequest.querystring;
			_restXML = new XML();
			_restXML.ignoreWhite = true;
			_restXML.onLoad = Delegate.create(this, onAPIResponse);
			LogWrapper.getLog().debug("Flashr.callMethod : "+url);
			_restXML.load(url);
			_lastQueryTimestamp = getTimer();
		}
	}
	
	/**
	* Function: onAPIResponse
	* Called internally on recepit of a response from a request to the
	* <REST_ENDPOINT>.
	* Interperets the received XML and broadcasts an onAPIResponse event to 
	* any registered listeners.
	* If the XML was the result of a known method call then the broadcast
	* event will include sensible information, otherwise it will just be the
	* XML for parsing outside this class (e.g. if a new method has been added
	* to the Flickr.com API and not programmed into this class).
	* 
	* Parameters:
	* success - Whether the call to the API was successfull (e.g. the server
	* specfied in <REST_ENDPOINT> was connected to) or not.
	* 
	**/
	private function onAPIResponse(success:Boolean):Void
	{
		awaitingResponse = false;
		var responseEvent:FlashrResponseEvent = new FlashrResponseEvent(_currentRequest);
		if (success) {
			LogWrapper.getLog().debug("Flashr.onAPIResponse received XML: "+_restXML);
			if (_restXML.firstChild.nodeName == "rsp") {
				_currentRequest.completed = true;
				if (_restXML.firstChild.attributes.stat == "ok") {
					/**
					* got a valid response - parse it based on the method which was called
					* if we don't know how to deal with this method then just return the
					* unparsed XML
					**/
					responseEvent.status = STATUS_OK;
					switch (_currentRequest.method) {
						case "flickr.auth.getFrob":
							_authFrob = _restXML.firstChild.firstChild.firstChild.nodeValue;
							if (_authIsLoggingIn) {
								_authOpenLogin();
								_authIsLoggingIn = false;
								// skip the rest of the function - the FlashrResponse 
								// doesn't need to get involved...
								return; 
							}
							responseEvent.data.frob = _authFrob;
							break;
						case "flickr.auth.getToken":
						case "flickr.auth.checkToken":
							var t:Array = _restXML.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<t.length; i++) {
								var thisNode:XMLNode = t[i];
								switch (thisNode.nodeName) {
									case "user":
										authUser = Person.getPerson(thisNode.attributes.nsid);
										authUser.username = thisNode.attributes.username;
										authUser.realname = thisNode.attributes.fullname;
										break;
									case "perms":
										authPerms = thisNode.firstChild.nodeValue;
										break;
									case "token":
										_authToken = thisNode.firstChild.nodeValue;
										break;
								}
							}
							responseEvent.data.token = _authToken;
							break;
						case "flickr.contacts.getList":
							var t:Array = _restXML.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<t.length; i++) {
								var attribs:Array = t[i].attributes;
								var p:Person = Person.getPerson(attribs.nsid);
								p.username = attribs.username;
								p.friend = attribs.friend;
								p.family = attribs.family;
								p.ignored = attribs.ignored;
								authUser.addPublicContact(p);
							}
							if (_currentRequest.additionalArguments.filter) {
								responseEvent.data.filter = _currentRequest.additionalArguments.filter;
								delete _currentRequest.additionalArguments.filter;
							}
							responseEvent.data.user = authUser;
							break;
						case "flickr.contacts.getPublicList":
							var user:Person = Person.getPerson(_currentRequest.additionalArguments.nsid);
							var t:Array = _restXML.firstChild.firstChild.childNodes;
							
							for (var i:Number=0; i<t.length; i++) {
								var attribs:Array = t[i].attributes;
								var contact:Person = Person.getPerson(attribs.nsid);
								contact.username = attribs.username;
								contact.friend = attribs.friend;
								contact.family = attribs.family;
								contact.ignored = attribs.ignored;
								user.addPublicContact(contact);
							}
							responseEvent.data.user = user;
							break;
						case "flickr.favorites.add":
							responseEvent.data.photoId = _currentRequest.additionalArguments.photoId;
							authUser.addFavorite(Photo.getPhoto(_currentRequest.additionalArguments.photoId));
							break;
						case "flickr.favorites.getList":
						case "flickr.favorites.getPublicList":
							var person:Person = Person.getPerson(_currentRequest.additionalArguments.userId);
							var photosList:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(photosList);
							var photos:Array = responseEvent.data.resultsSet.photos;
							for(var i:Number=0; i<photos.length; i++) {
								person.addFavorite(photos[i]);
							}
							responseEvent.data.user = person;
							break;
						case "flickr.favorites.remove":
							responseEvent.data.photoId = _currentRequest.additionalArguments.photoId;
							authUser.removeFavorite(Photo.getPhoto(_currentRequest.additionalArguments.photoId));
							break;
						case "flickr.groups.pools.add":
							var photo:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var group:Group = Group.getGroup(_currentRequest.additionalArguments.groupId);
							group.addPhoto(photo);
							responseEvent.data.photo = photo;
							responseEvent.data.group = group;
							break;
						case "flickr.groups.pools.getContext":
							var contextXml:XMLNode = _restXML.firstChild;
							var photo:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var group:Group = Group.getGroup(_currentRequest.additionalArguments.groupId);
							var parsedContext:Object = _parseContextXml(contextXml);
							group.numPhotos = parsedContext.count;
							group.addPhoto(parsedContext["prevphoto"]);
							group.addPhoto(photo);
							group.addPhoto(parsedContext["nextphoto"]);
							photo.setContextGroupPool(group.nsid, parsedContext["prevphoto"], parsedContext["nextphoto"]);
							responseEvent.data.photo = photo;
							responseEvent.data.group = group;
							break;
						case "flickr.groups.pools.getGroups":
							var groupsXml:XMLNode = _restXML.firstChild.firstChild;
							for (var i:Number=0; i<groupsXml.childNodes.length; i++) {
								var g:Group = Group.getGroup(groupsXml.childNodes[i].attributes.nsid);
								g.name = groupsXml.childNodes[i].attributes.name;
								// TODO: what about the admin attribute? It shouldn't go on the group because it describes the relationship between the calling <Person> and the <Group> not the <Group> itself.
								g.privacy = groupsXml.childNodes[i].attributes.privacy;
								g.numPhotos = groupsXml.childNodes[i].attributes.photos;
								authUser.addGroup(g);
							}
							responseEvent.data.user = authUser;
							break;
						case "flickr.groups.pools.getPhotos":
							var group:Group = Group.getGroup(_currentRequest.additionalArguments.groupId);
							responseEvent.data.resultsSet = _parsePhotoXml(_restXML.firstChild.firstChild);
							var photos:Array = responseEvent.data.resultsSet.photos;
							for (var i:Number=0; i<photos.length; i++) {
								// TODO: this isn't right - we should somehow be adding the photos in to the
								// correct place in the <Group._pool>. We should also be setting the correct <Group.numPhotos>
								group.addPhoto(photos[i]);
							}
							responseEvent.data.group = group;
							break;
						case "flickr.groups.pools.remove":
							var photo:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var group:Group = Group.getGroup(_currentRequest.additionalArguments.groupId);
							group.removePhoto(photo);
							responseEvent.data.photo = photo;
							responseEvent.data.group = group;
							break;
						case "flickr.interestingness.getList":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							responseEvent.data.date = _currentRequest.additionalArguments.date == undefined ? new Date() : new Date(_currentRequest.additionalArguments.date.getFullYear(), _currentRequest.additionalArguments.date.getMonth(), _currentRequest.additionalArguments.date.getDate());
							break;
						case "flickr.people.findByEmail":
						case "flickr.people.findByUsername":
							var p:Person = Person.getPerson(_restXML.firstChild.firstChild.attributes.nsid);
							p.username = _restXML.firstChild.firstChild.firstChild.firstChild.nodeValue;
							if (_currentRequest.method == "flickr.people.findByEmail") {
								p.email = _currentRequest.additionalArguments.email;
							}
							responseEvent.data.person = p;
							break;
						case "flickr.people.getInfo":
							var px:XMLNode = _restXML.firstChild.firstChild;
							var p:Person = Person.getPerson(px.attributes.nsid);
							p.isAdmin = px.attributes.isadmin;
							p.isPro = px.attributes.ispro;
							p.iconServer = px.attributes.iconserver;
							for (var i:Number=0; i<px.childNodes.length; i++) {
								var thisNode:XMLNode = px.childNodes[i];
								switch (thisNode.nodeName) {
									case "username":
										p.username = thisNode.firstChild.nodeValue;
										break;
									case "realname":
										p.realname = thisNode.firstChild.nodeValue;
										break;
									case "location":
										p.location = thisNode.firstChild.nodeValue;
										break;
									case "mbox_sha1sum":
										p.mboxSha1sum = thisNode.firstChild.nodeValue;
										break;
									case "photos":
										for(var j:Number=0; j<thisNode.childNodes.length; j++) {
											switch (thisNode.childNodes[j].nodeName) {
												case "firstdatetaken":
													p.photosFirstDateTaken = DateFormat.isoToDate(thisNode.childNodes[j].firstChild.nodeValue);
													break;
												case "firstdate":
													// date constructor expects number of milliseconds since 1970 - Flickr API returns UNIX timestamp = number seconds since 1970
													var d:Date = DateFormat.secsToDate(thisNode.childNodes[j].firstChild.nodeValue);
													p.photosFirstDate = d;
													break;
												case "count":
													p.numPhotos = thisNode.childNodes[j].firstChild.nodeValue;
													break;
											}
										}
										break;
								}
							}
							responseEvent.data.person = p;
							break;
						case "flickr.people.getPublicGroups":
							responseEvent.data.person = Person.getPerson(_currentRequest.additionalArguments.nsid);
							var groups:Array = _restXML.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<groups.length; i++) {
								var groupData:Object = groups[i].attributes;
								var g:Group = Group.getGroup(groupData.nsid);
								g.name = groupData.name;
								g.eighteenPlus = groupData.eighteenplus;
								responseEvent.data.person.addPublicGroup(g, groupData.admin);
							}
							break;
						case "flickr.people.getPublicPhotos":
							/**
							* TODO:
							* Do we need to deal with the pagination info somehow here?
							* Or is it up to the application programmer to keep track of what s/he
							* asked for? But if more photos are added to the photos array simply with
							* addPhoto then how can the application programmer know which ones they
							* should be interested in?
							**/ 
							var thisPerson:Person = Person.getPerson(_currentRequest.additionalArguments.nsid);
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							var photos:Array = responseEvent.data.resultsSet.photos;
							for (var i:Number=0; i<photos.length; i++) {
								thisPerson.addPhoto(photos[i]);
							}
							responseEvent.data.person = thisPerson;
							break;
						case "flickr.people.getUploadStatus":
							var t:Array = _restXML.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<t.length; i++) {
								var thisNode:XMLNode = t[i];
								switch (thisNode.nodeName) {
									case "username":
										// we must already know the username as it is returned when we log in
										// and we must be logged in to be hear so lets just ignore it!
										break;
									case "bandwidth":
										authUser.bandwidthMax = thisNode.attributes.max;
										authUser.bandwidthUsed = thisNode.attributes.used;
										break;
									case "filesize":
										authUser.filesizeMax = thisNode.attributes.max;
										break;
								}
							}
							responseEvent.data.user = authUser;
							break;
						case "flickr.photos.addTags":
							responseEvent.data.photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							/**
							* TODO:
							* Should we add the tags to <Photo._tags> here? Probably can't as 
							* we don't have as much info as we need e.g. the ids of the tags
							* that have been added or the nsid of the calling user.
							* But you would kind of expect them to be added...
							**/
							responseEvent.data.tags = _currentRequest.additionalArguments.tags;
							break;
						case "flickr.photos.getAllContexts":
							var thisPhoto = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var nodes:Array = _restXML.firstChild.childNodes;
							for(var i:Number=0; i<nodes.length; i++) {
								switch (nodes[i].nodeName) {
									case "set":
										var ps:Photoset = Photoset.getPhotoset(nodes[i].attributes.id);
										ps.title = nodes[i].attributes.title;
										ps.addPhoto(thisPhoto);
										// TODO this!
										break;
									case "pool":
										// TODO this!
										break;
									default:
										LogWrapper.getLog().error("flickr.photos.getAllContexts returned node "+nodes[i].nodeName+" - what is it???");
								}
							}
							responseEvent.data.photo = thisPhoto;
							break;
						case "flickr.photos.getContactsPhotos":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							// add any of the contacts found through this as public contacts to auth'd user
							for (var i:Number=0; i<responseEvent.data.contactsPhotos.length; i++) {
								authUser.addPublicContact(responseEvent.data.contactsPhotos[i].owner);
							}
							break;
						case "flickr.photos.getContactsPublicPhotos":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							responseEvent.data.user = Person.getPerson(_currentRequest.additionalArguments.nsid);
							// add any of the contacts found through this as public contacts to user...
							for (var i:Number=0; i<responseEvent.data.contactsPhotos.length; i++) {
								responseEvent.data.user.addPublicContact(responseEvent.data.contactsPhotos[i].owner);
							}
							break;
						case "flickr.photos.getContext":
							var thisPhoto = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var nodes:Array = _restXML.firstChild.childNodes;
							var parsedContext = _parseContextXml(_restXML.firstChild);
							// TODO: should we try and add these photos to their owners _photos Array if we know who the owner is?
							thisPhoto.contextCount = parsedContext.count;
							thisPhoto.contextPhotoPrevious = parsedContext.prevphoto;
							thisPhoto.contextPhotoNext = parsedContext.nextphoto;
							responseEvent.data.thisPhoto = thisPhoto;
							break;
						case "flickr.photos.getCounts":
							var nodes:Array = _restXML.firstChild.firstChild.childNodes;
							responseEvent.data.photoCounts = [];
							for(var i:Number=0; i<nodes.length; i++) {
								var countObj:Object = {};
								for(var attributeName:String in nodes[i].attributes) {
									switch (attributeName) {
										case "count":
											countObj[attributeName] = nodes[i].attributes[attributeName];
											break;
										case "fromdate":
										case "todate":
											switch (_currentRequest.additionalArguments.dateType) {
												case "uploaded":
													countObj[attributeName] = DateFormat.secsToDate(nodes[i].attributes[attributeName]);
													break;
												case "taken":
													countObj[attributeName] = DateFormat.isoToDate(nodes[i].attributes[attributeName]);
													break;
											}
									}
								}
								responseEvent.data.photoCounts.push(countObj);
							}
							break;
						case "flickr.photos.getExif":
							var photoInfo:Object = _restXML.firstChild.firstChild.attributes;
							responseEvent.data.photo = Photo.getPhoto(photoInfo.id);
							responseEvent.data.photo.secret = photoInfo.secret;
							responseEvent.data.photo.server = photoInfo.server;
							var exifNodes:Array = _restXML.firstChild.firstChild.childNodes;
							responseEvent.data.photo.exifData = [];
							for (var i:Number=0; i<exifNodes.length; i++) {
								var attr:Object = exifNodes[i].attributes;
								var e:ExifData = new ExifData();
								e.tagspace = attr.tagspace;
								e.tagspaceid = attr.tagspaceid;
								e.tag = attr.tag;
								e.label = attr.label;
								
								var subNodes:Array = exifNodes[i].childNodes;
								for(var j:Number=0; j<subNodes.length; j++) {
									e[subNodes[j].nodeName] = subNodes[j].firstChild.nodeValue;
								}
								
								responseEvent.data.photo.exifData.push (e);
							}
							break;
						case "flickr.photos.getInfo":
							var photoInfo:Object = _restXML.firstChild.firstChild.attributes;
							var ph:Photo = Photo.getPhoto(photoInfo.id);
							ph.secret = photoInfo.secret;
							ph.server = photoInfo.server;
							ph.getInfoCalled = true;
							if (photoInfo.isfavorite && photoInfo.isfavorite>0) {
								authUser.addFavorite(ph);
							}
							ph.licence = photoInfo.license;
							ph.rotation = photoInfo.rotation;
							
							var additionalInfo:Array = _restXML.firstChild.firstChild.childNodes;
							
							for (var i:Number=0; i<additionalInfo.length; i++) {
								var thisNode:XMLNode = additionalInfo[i];
								switch (thisNode.nodeName) {
									case "owner":
										var person:Person = Person.getPerson(thisNode.attributes.nsid);
										person.username = thisNode.attributes.username;
										person.realname = thisNode.attributes.realname;
										person.location = thisNode.attributes.location;
										ph.owner = person;
										person.addPhoto(ph);
										break;
									case "title":
										ph.title = thisNode.firstChild.nodeValue;
										break;
									case "description":
										ph.description = thisNode.firstChild.nodeValue;
										break;
									case "visibility":
										ph.isPublic = thisNode.attributes.ispublic;
										// TODO - again these are related to the calling user as well
										// as to the photo itself - where to store the info??
										ph.isFriend = thisNode.attributes.isfriend;
										ph.isFamily = thisNode.attributes.isfamily;
										break;
									case "dates":
										ph.dateUploaded = thisNode.attributes.posted;
										ph.dateTaken = thisNode.attributes.taken;
										// TODO - what is thisNode.attributes.takengranularity ??
										ph.dateUpdated = thisNode.attributes.lastupdate;
										break;
									case "permissions":
										// TODO - again these are related to the calling user as well
										// as to the photo itself - where to store the info??
										break;
									case "editability":
										/**
										* TODO - somehow need to associate this information with
										* the calling user as opposed to the photo or the photo's
										* owner...
										* This is easier now as we have authUser BUT do we just store the
										* info on the photo or do we store it on the user keyed to the 
										* photo id?
										**/
										break;
									case "comments":
										ph.numComments = Number(thisNode.firstChild.nodeValue);
										break;
									case "notes":
										var noteNodes:Array = thisNode.childNodes;
										for (var j:Number=0; j<noteNodes.length; j++) {
											var n:Note = Note.getNote(noteNodes[j].attributes.id);
											n.x = noteNodes[j].attributes.x;
											n.y = noteNodes[j].attributes.y;
											n.width = noteNodes[j].attributes.w;
											n.height = noteNodes[j].attributes.h;
											n.text = noteNodes[j].firstChild.nodeValue;
											var noteAuthor:Person = Person.getPerson(noteNodes[j].attributes.author);
											noteAuthor.username = noteNodes[j].attributes.authorname;
											n.author = noteAuthor;
											ph.addNote(n);
										}
										break;
									case "tags":
										var tagNodes:Array = thisNode.childNodes;
										for (var j:Number=0; j<tagNodes.length; j++) {
											var t:Tag = Tag.getTag(tagNodes[j].attributes.raw);
											var author:Person = Person.getPerson(tagNodes[j].attributes.author);
											t.value = tagNodes[j].firstChild.nodeValue;
											ph.addTag(tagNodes[j].attributes.id, Tag(t), author);
											t.addPhoto(tagNodes[j].attributes.id, ph, author);
										}
										break;
									case "urls":
										var urlNodes:Array = thisNode.childNodes;
										for (var j:Number=0; j<urlNodes.length; j++) {
											switch (urlNodes[j].attributes.type) {
												case "photopage":
													ph.photoPageUrl = urlNodes[j].firstChild.nodeValue;
													break;
												default:
													LogWrapper.getLog().error("flickr.photos.getInfo - what is a subnode of the urls tag called \""+urlNodes[j].attributes.type+"\"?");
											}
										}
										break;
									default:
										LogWrapper.getLog().error("flickr.photos.getInfo - what is a node called \""+thisNode.nodeName+"\"?");
								}
							}
							responseEvent.data.photo = ph;
							break;
						case "flickr.photos.getNotInSet":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							break;
						case "flickr.photos.getPerms":
							var px:XMLNode = _restXML.firstChild.firstChild;
							var photo:Photo = Photo.getPhoto(px.attributes.id);
							photo.isPublic = px.attributes.ispublic;
							/**
							* TODO:
							* Need to associate the following attributes with the calling user?
							* Should this be done in the _parsePhotoXML function even though there is
							* only one photo?
							**/
							//photo.isFriend = px.attributes.isfriend;
							//photo.isFamily = px.attributes.isfamily;
							//photo.permComment = px.attributes.permcomment;
							//photo.permAddMeta = px.attributes.permaddmeta;
							responseEvent.data.photo = photo;
							break;
						case "flickr.photos.getRecent":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							break;
						case "flickr.photos.getSizes":
							var thisPhoto:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var sx:XMLNode = _restXML.firstChild.firstChild;
							for (var i:Number=0; i<sx.childNodes.length; i++) {
								var attribs:Object = sx.childNodes[i].attributes;
								var thisSize:PhotoSize = new PhotoSize(thisPhoto, attribs.label);
								thisSize.width = attribs.width;
								thisSize.height = attribs.height;
								thisSize.source = attribs.source;
								thisSize.url = attribs.url;
								thisSize.label = attribs.label;
								thisPhoto.addSize(thisSize);
								if (attribs.label == "Original") {
									thisPhoto.originalUrl = attribs.source;
								}
							}
							responseEvent.data.photo = thisPhoto;
							break;
						case "flickr.photos.getUntagged":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							break;
						case "flickr.photos.removeTag":
							/**
							* nothing is returned... just forward on id of the tag that was removed
							*
							* if we could map the email address to a <Person> then we could 
							* remove the <Tag> from <Person._tags>. We can't remove the <Tag> from 
							* the relevant <Person> or <Photo> because we only know the id and not
							* the Raw which we would need to create a <Tag> instance...
							**/
							responseEvent.data.tagId = _currentRequest.additionalArguments.tagId;
							break;
						case "flickr.photos.search":
							var px:XMLNode = _restXML.firstChild.firstChild;
							responseEvent.data.resultsSet = _parsePhotoXml(px);
							break;
						case "flickr.photos.setDates":
							responseEvent.data.photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							if (_currentRequest.additionalArguments.dateTaken != undefined) responseEvent.data.photo.dateTaken = _currentRequest.additionalArguments.dateTaken;
							if (_currentRequest.additionalArguments.datePosted != undefined) responseEvent.data.photo.dateUploaded = _currentRequest.additionalArguments.datePosted;
							break;
						case "flickr.photos.setMeta":
							responseEvent.data.photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							responseEvent.data.photo.title = _currentRequest.additionalArguments.title;
							responseEvent.data.photo.description = _currentRequest.additionalArguments.description;
							break;
						case "flickr.photos.setPerms":
							/**
							* TODO:
							* Should probably update the permissions info somewhere on the <Photo>
							* or in the relationship between that and the logged in user?
							**/
							responseEvent.data.photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							break;
						case "flickr.photos.setTags":
							responseEvent.data.photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							responseEvent.data.photo.setTags(_currentRequest.additionalArguments.tags, authUser);
							break;
						case "flickr.photos.comments.getList":
							var px:XMLNode = _restXML.firstChild.firstChild;
							var ph:Photo = Photo.getPhoto(px.attributes.photo_id);
							for (var i:Number=0; i<px.childNodes.length; i++) {
								var thisCommentNode:XMLNode = px.childNodes[i];
								var c:Comment = Comment.getComment(thisCommentNode.attributes.id);
								c.author = Person.getPerson(thisCommentNode.attributes.author);
								c.created = DateFormat.secsToDate(thisCommentNode.attributes.datecreate);
								c.url = thisCommentNode.attributes.permalink;
								c.comment = thisCommentNode.firstChild.nodeValue;
								ph.addComment(c);
							}
							responseEvent.data.photo = ph;
							break;
						case "flickr.photosets.addPhoto":
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							var photo:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							ps.addPhoto(photo);
							
							// if this call was successful then both the photo and the photoset are owned by 
							// the authUser so we should reflect this...
							photo.owner = authUser;
							authUser.addPhoto(photo);
							authUser.addPhotoset(ps);
							ps.owner = authUser;
							
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.create":
							var photosetXml:XMLNode = _restXML.firstChild.firstChild;
							var ps:Photoset = Photoset.getPhotoset(photosetXml.attributes.id);
							var primaryPhoto:Photo = Photo.getPhoto(_currentRequest.additionalArguments.primaryPhotoId);
							ps.url = photosetXml.attributes.url;
							ps.owner = authUser;
							ps.title = _currentRequest.additionalArguments.title;
							ps.description = _currentRequest.additionalArguments.description;
							ps.addPhoto(primaryPhoto);
							authUser.addPhotoset(ps);
							authUser.addPhoto(primaryPhoto);
							
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.delete":
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							responseEvent.data.photosetId = ps.id;
							authUser.removePhotoset(ps);
							Photoset.remove(ps);
							
							break;
						case "flickr.photosets.editMeta":
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							ps.title = _currentRequest.additionalArguments.title;
							ps.description = _currentRequest.additionalArguments.description;
							ps.owner = authUser;
							authUser.addPhotoset(ps);
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.editPhotos":
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							var primaryPhoto:Photo = Photo.getPhoto(_currentRequest.additionalArguments.primaryPhotoId);
							ps.owner = authUser;
							ps.editPhotos(primaryPhoto, _currentRequest.additionalArguments.photoIds);
							authUser.addPhotoset(ps);
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.getContext":
							var contextXml:XMLNode = _restXML.firstChild;
							var photo:Photo = Photo.getPhoto(_currentRequest.additionalArguments.photoId);
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							var parsedContext:Object = _parseContextXml(contextXml);
							photo.setContextPhotoset(_currentRequest.additionalArguments.photosetId, parsedContext["prevphoto"], parsedContext["nextphoto"]);
							
							// TODO: this doesn't necessarily put these photos in the right bit of the <Photoset.photo>s array
							ps.addPhoto(parsedContext["prevphoto"]);
							ps.addPhoto(photo);
							ps.addPhoto(parsedContext["nextphoto"]);
							
							responseEvent.data.photo = photo;
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.getInfo":
							responseEvent.data.photoset = _parsePhotosetXml(_restXML.firstChild.firstChild);
							break;
						case "flickr.photosets.getList":
							var owner:Person;
							if (_currentRequest.additionalArguments.userId) {
								owner = Person.getPerson(_currentRequest.additionalArguments.userId);
							} else {
								owner = authUser;
							}
							// TODO: Should we blank the user's photosets array since we know we are getting the definitive list here?
							var photosetsXml:XMLNode = _restXML.firstChild.firstChild;
							for (var i:Number=0; i<photosetsXml.childNodes.length; i++) {
								var ps:Photoset = _parsePhotosetXml(photosetsXml.childNodes[i], owner);
								owner.addPhotoset(ps);
							}
							responseEvent.data.photosetOwner = owner;
							break;
						case "flickr.photosets.getPhotos":
							var photosetXml:XMLNode = _restXML.firstChild.firstChild;
							var ps:Photoset = Photoset.getPhotoset(photosetXml.attributes.id);
							ps.primaryPhoto = Photo.getPhoto(photosetXml.attributes.primary);
							ps.clearPhotos();
							for (var i:Number=0; i<photosetXml.childNodes.length; i++) {
								var photoInfo:Object = photosetXml.childNodes[i].attributes;
								var photo:Photo = Photo.getPhoto(photoInfo.id);
								photo.secret = photoInfo.secret;
								photo.server = photoInfo.server;
								photo.title = photoInfo.title;
								if (photoInfo.license !== undefined) {
									photo.licence = photoInfo.license;
								}
								if (photoInfo.dateupload != undefined) {
									photo.dateUploaded = photoInfo.dateupload;
								}
								if (photoInfo.datetaken != undefined) {
									photo.dateTaken = photoInfo.datetaken;
									// TODO ignoring attribs.datetakengranularity
								}
								if (photoInfo.originalformat != undefined) {
									photo.originalFormat = photoInfo.originalformat;
								}
								// can ignore photoInfo.isprimary because we already know the primary from above
								ps.addPhoto(photo);
							}
							responseEvent.data.photoset = ps;
							break;
						case "flickr.photosets.orderSets":
							// TODO: this assumes that you have passed all the photoset id's to do the ordering
							// but it is valid to leave photoset id's out and then they are ordered by their id's
							// so the user's photosets list is not necessarily correct after a call to this method
							// How to deal with this?
							authUser.clearPhotosets();
							for (var i:Number=0; i<_currentRequest.additionalArguments.photosetIds.length; i++) {
								authUser.addPhotoset(Photoset.getPhotoset(_currentRequest.additionalArguments.photosetIds[i]));
							}
							responseEvent.data.user = authUser;
							break;
						case "flickr.photosets.removePhoto":
							var ps:Photoset = Photoset.getPhotoset(_currentRequest.additionalArguments.photosetId);
							ps.removePhoto(Photo.getPhoto(_currentRequest.additionalArguments.photoId));
							responseEvent.data.photoset = ps;
							break;
						case "flickr.tags.getListPhoto":
							responseEvent.data.photo = Photo.getPhoto(_restXML.firstChild.firstChild.attributes["id"]);
							var t:Array = _restXML.firstChild.firstChild.firstChild.childNodes;
							var authors:Object = {};
							for (var i:Number=0; i<t.length; i++) {
								if (authors[t[i].attributes.author] == undefined) {
									authors[t[i].attributes.author] = Person.getPerson(t[i].attributes.author);
									authors[t[i].attributes.author].username = t[i].attributes.authorname;
									
								}
								var ta:Tag = Tag.getTag(t[i].attributes.raw);
								ta.value = t[i].firstChild.nodeValue == undefined ? t[i].attributes.raw : t[i].firstChild.nodeValue;
								ta.addPhoto(t[i].attributes.id, responseEvent.data.photo, authors[t[i].attributes.author]);
								//responseEvent.data.photo.addTag(ta.raw, ta, authors[t[i].attributes.author]);
								responseEvent.data.photo.addTag(t[i].attributes.id, ta, authors[t[i].attributes.author]);
							}
							break;
						case "flickr.tags.getListUser":
							var tagUserId:String = _restXML.firstChild.firstChild.attributes["id"];
							responseEvent.data.person = Person.getPerson(tagUserId);
							var t:Array = _restXML.firstChild.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<t.length; i++) {
								var ta:Tag = Tag.getTag(t[i].firstChild.nodeValue);
								responseEvent.data.person.addTag(ta);
							}
							break;
						case "flickr.tags.getListUserPopular":
							var tagUserId:String = _restXML.firstChild.firstChild.attributes["id"];
							responseEvent.data.person = Person.getPerson(tagUserId);
							var t:Array = _restXML.firstChild.firstChild.firstChild.childNodes;
							for (var i:Number=0; i<t.length; i++) {
								var ta:Tag = Tag.getTag(t[i].firstChild.nodeValue);
								responseEvent.data.person.addTag(ta, t[i].attributes.count);
							}
							break;
						case "flickr.tags.getRelated":
							var t:Array = _restXML.firstChild.firstChild.childNodes;
							var sourceTag:Tag = Tag.getTag(_restXML.firstChild.firstChild.attributes.source);
							for (var i:Number=0; i<t.length; i++) {
								sourceTag.addRelatedTag(Tag.getTag(t[i].firstChild.nodeValue));
							}
							responseEvent.data.sourceTag = sourceTag;
							break;
						case "flickr.test.echo":
							responseEvent.data.params = [];
							var echo:Array = _restXML.firstChild.childNodes;
							for (var i:Number=0; i<echo.length; i++) {
								responseEvent.data.params.push({name:echo[i].nodeName, value:echo[i].firstChild.nodeValue});
							}
							break;
						case "flickr.test.login":
							responseEvent.data.user = authUser;
							break;
						default:
							responseEvent.data.restXML = _restXML;
							break;
					}
					if (cacheQueries) {
						cache.cache(responseEvent);
					}
				} else if (_restXML.firstChild.attributes.stat == "fail") {
					responseEvent.status = Number(_restXML.firstChild.firstChild.attributes.code);
					responseEvent.errorMessage = _restXML.firstChild.firstChild.attributes.msg;
				}
			} else {
				// TODO: Does this make sense?
				// requeue the request presuming the problem was temporary...
				queueRequest(_currentRequest);
				
				LogWrapper.getLog().error("Invalid XML returned - outermost node: "+_restXML.firstChild.nodeName);
				responseEvent.status = STATUS_INVALID_XML;
				responseEvent.errorMessage = STATUS_INVALID_XML_DESCRIPTION;
			}
		} else {
			// TODO: Does this make sense?
			// requeue the request presuming the problem was temporary...
			queueRequest(_currentRequest);
			
			LogWrapper.getLog().fatal("Error connecting to server");
			responseEvent.status = STATUS_ERROR_CONNECTING;
			responseEvent.errorMessage = STATUS_ERROR_CONNECTING_DESCRIPTION;
		}
		dispatchEvent(responseEvent);
		if (saveHistory) {
			history.addResponse(responseEvent);
		}
		_executeNextRequest();
	}
	
	public function set REST_ENDPOINT(REST_ENDPOINT:String):Void
	{
		LogWrapper.getLog().debug("REST_ENDPOINT: "+REST_ENDPOINT);
		_REST_ENDPOINT = REST_ENDPOINT;
	}
	public function get REST_ENDPOINT():String
	{
		return _REST_ENDPOINT;
	}
	
	public function get apiKey():String
	{
		return _apiKey;
	}
	public function set apiKey(apiKey:String):Void
	{
		//LogWrapper.getLog().debug("API Key set to: "+apiKey);
		// we won't output our whole key for the world to see...
		LogWrapper.getLog().debug("API Key set to: " + apiKey.substr(0, -5) + "*****");
		_apiKey = apiKey;
	}
	
	/**
	* Function: _parsePhotoXML
	* 
	* Private internal function used to avoid duplicate code to 
	* parse "the standard photo list xml"
	* 
	* Params:
	* px			-	The "standard photo list xml" from flickr.
	* 
	* Returns:
	* A <ResultsSet>.
	**/
	private function _parsePhotoXml(px:XMLNode):ResultsSet
	{
		/**
		* TODO:
		* Done?
		**/
		var resultsSet:ResultsSet = new ResultsSet();
		resultsSet.page = Number(px.attributes.page);
		resultsSet.numberOfPages = Number(px.attributes.pages);
		resultsSet.resultsPerPage = Number(px.attributes.perpage);
		resultsSet.total = Number(px.attributes.total);
		// TODO: Should probably take the api_sig and key etc out of this URL?
		resultsSet.request = _currentRequest;
		var photos:Array = [];
		var foundPeople:Object = {}; // so we only create each contact once...
		for (var i:Number=0; i<px.childNodes.length; i++) {
			var attribs:Object = px.childNodes[i].attributes;
			if (foundPeople[attribs.owner] == undefined) {
				foundPeople[attribs.owner] = Person.getPerson(attribs.owner);
				if (attribs.ownername != undefined) {
					foundPeople[attribs.owner].username = attribs.ownername;
				}
				if (attribs.username != undefined) {
					foundPeople[attribs.owner].username = attribs.username;
				}
				if (attribs.iconserver != undefined) {
					foundPeople[attribs.owner].iconServer = attribs.iconserver;
				}
			} 
			var ph:Photo = Photo.getPhoto(attribs.id);
			ph.owner = foundPeople[attribs.owner];
			ph.secret = attribs.secret;
			ph.server = attribs.server;
			ph.title = attribs.title;
			ph.isPublic = attribs.ispublic;
			if (attribs.license !== undefined) {
				ph.licence = attribs.license;
			}
			if (attribs.dateupload != undefined) {
				ph.dateUploaded = attribs.dateupload;
			}
			if (attribs.datetaken != undefined) {
				ph.dateTaken = attribs.datetaken;
				// TODO ignoring attribs.datetakengranularity
			}
			if (attribs.originalformat != undefined) {
				ph.originalFormat = attribs.originalformat;
			}
			/**
			* TODO:
			* What about
			* attribs.isfriend
			* attribs.isfamily
			* ?? 
			**/
			foundPeople[attribs.owner].addPhoto(ph);
			photos.push(ph);
		}
		resultsSet.photos = photos;
		return resultsSet;
	}
	/**
	* Function: _parsePhotosetXml
	* 
	* Private internal function used to avoid duplicate code when parsing the
	* standard photoset XML
	* 
	* Parameters:
	* photosetXml				-	An XML node describing a photoset.
	* owner						-	The owner of this <Photoset> - leave as undefined if they exist as
	* 								an attribute in the XML...
	* 
	* Returns:
	* The created <Photoset>.
	**/
	private function _parsePhotosetXml(photosetXml:XMLNode, owner:Person):Photoset
	{
			if (owner == undefined) {
				owner = Person.getPerson(photosetXml.attributes.owner);
			}
			var ps:Photoset = Photoset.getPhotoset(photosetXml.attributes.id);
			var primaryPhoto:Photo = Photo.getPhoto(photosetXml.attributes.primary);
			ps.owner = owner;
			ps.primaryPhoto = primaryPhoto;
			ps.addPhoto(primaryPhoto);
			owner.addPhoto(primaryPhoto);
			primaryPhoto.owner = owner;
			primaryPhoto.secret = photosetXml.attributes.secret;
			if (photosetXml.attributes.server) {
				primaryPhoto.server = photosetXml.attributes.server;
			}
			ps.numPhotos = photosetXml.attributes.photos;
			for (var i:Number=0; i<photosetXml.childNodes.length; i++) {
				var thisNode:XMLNode = photosetXml.childNodes[i];
				switch(thisNode.nodeName) {
					case "title":
						ps.title = thisNode.firstChild.nodeValue;
						break;
					case "description":
						ps.description = thisNode.firstChild.nodeValue;
						break;
				}
			}
			return ps;
	}
	
	/**
	* Function: _parseContextXml
	* 
	* Private internal function used to avoid duplicate code when parsing the
	* standard context XML
	* 
	* Parameters:
	* contextXML				-	The contextXML to parse
	* 
	* Returns:
	* An object containing count:Number, prevphoto:Photo and nextphoto:Photo
	**/
	private function _parseContextXml(contextXML:XMLNode):Object
	{
		var returnObj:Object = {};
		for (var i:Number=0; i<contextXML.childNodes.length; i++) {
			switch(contextXML.childNodes[i].nodeName) {
				case "count":
					returnObj.count = contextXML.childNodes[i].firstChild.nodeValue;
					break;
				case "prevphoto":
				case "nextphoto":
					var photoInfo:Object = contextXML.childNodes[i].attributes;
					var thisPhoto:Photo;
					if (photoInfo.id > 0) {
						thisPhoto = Photo.getPhoto(photoInfo.id);
						thisPhoto.secret = photoInfo.secret;
						thisPhoto.title = photoInfo.title;
						// TODO: How to store the URL, it is the URL of the photo in context - is it useful??
						thisPhoto.thumbnailUrl = photoInfo.thumb;
						returnObj[contextXML.childNodes[i].nodeName] = thisPhoto;
					}
					break;
			}
		}
		return returnObj;
	}
	
	/**
	* Function: getFlashr
	* 
	* Returns a reference to the Flashr instance -
	* if none exists yet then creates one and returns a reference to that.
	* 
	* Returns:
	* 
	* A <Flashr> instance
	**/
	public static function getFlashr():Flashr
	{
		if (_flashr == null) {
			_flashr = new Flashr();
		}
		return _flashr;
	}
	
	public function toString():String
	{
		return "[Object com.kelvinluck.flashr.core.Flashr]";
	}
	// function created by EventDispatcher
	function dispatchEvent() {};
	// function created by EventDispatcher
 	function addEventListener() {};
	// function created by EventDispatcher
 	function removeEventListener() {};
}