import com.kelvinluck.flashr.core.FlashrRequest;
import com.kelvinluck.util.LogWrapper;
import com.shockwave_india.util.PriorityQueue;
/**
* Class: com.kelvinluck.flashr.core.FlashrRequestQueue
* Handles the queue of <FlashrRequest> objects sent to the Flickr API.
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.FlashrRequestQueue 
{
	
	private static var _instance:FlashrRequestQueue;
	
	private var _priorityQueue:PriorityQueue;
	
	/**
	* Function: FlashrRequestQueue
	* Private constructor &ndash; There should only be one queue so use <getInstance> to get it...
	**/
	private function FlashrRequestQueue()
	{
		_priorityQueue = new PriorityQueue();
	}
	
	/**
	* Function: clearQueue
	* Empties the queue.
	**/
	function clearQueue()
	{
		_priorityQueue.init();
		LogWrapper.getLog().info("FlashrRequestQueue cleared");
	}
	/**
	* Function: getNext
	* Get the next <FlashrRequest> Object to be sent to flickr.com.
	* 
	* Returns:
	* The next <FlashrRequest> Object to be sent to flickr.com.
	**/
	function getNext():FlashrRequest
	{
		return FlashrRequest(_priorityQueue.removeTop());
	}
	
	/**
	* Function: addToQueue
	* Adds a <FlashrRequest> to the queue.
	* 
	* Parameters:
	* request					-	The <FlashrRequest> you want to add to the queue.
	* priority					-	The priority of this request. 
	**/
	function addToQueue(request:FlashrRequest, priority:Number) 
	{
		request.queueId = _priorityQueue.insert(request, priority);
	}
	
	/**
	* Function: removeFromQueue
	* Removes a given request from the queue.
	* 
	* Parameters:
	* The <FlashrRequest> you want to remove.
	* 
	* Returns:
	* True if the request was found and removed, false otherwise.
	**/
	function removeFromQueue(request:FlashrRequest):Boolean
	{
		return _priorityQueue.remove(request.queueId);
	}
	
	/**
	* Function: isEmpty
	* Tells you whether the request queue is empty or not.
	* 
	* Returns:
	* True if the queue is empty, false if there is still stuff in it...
	**/
	function isEmpty():Boolean
	{
		return _priorityQueue.getTopPriority() == undefined;
	}
	
	/**
	* Function: getInstance
	* Gets a reference to the Singleton instance of this class.
	* 
	* Returns:
	* A reference to the one and only <FlashrRequestQueue>.
	**/
	public static function getInstance():FlashrRequestQueue
	{
		if (_instance == null) {
			_instance = new FlashrRequestQueue();
		}
		return _instance;
	}
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.core.FlashrRequestQueue]";
	}
} 