import com.kelvinluck.flashr.core.FlashrRequest;
/**
* Class: FlashrResponseEvent
* Class to hold all the information that is dispatched as an event from <Flashr> when there is
* a response to a request. These events are caught by <FlashrResponse> which decodes them and calls the
* relevant method on itself.
* 
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.FlashrResponseEvent 
{
	
	/**
	* Variable: status
	* The status of the response.
	* 
	* See Also:
	* <Flashr.STATUS_BUSY>
	* 
	* <Flashr.STATUS_ERROR_CONNECTING>
	* 
	* <Flashr.STATUS_INVALID_XML>
	* 
	* <Flashr.STATUS_USER_NOT_LOGGED_IN>
	* 
	* <Flashr.STATUS_INVALID_API_KEY>
	* 
	* <Flashr.STATUS_OK>
	**/
	var status:Number;
	
	/**
	* Variable: request
	* The <FlashrRequest> that led to this response.
	**/
	var request:FlashrRequest;
	
	/**
	* Variable: type 
	* The type of this event (always onAPIResponse)
	**/
	var type:String = "onAPIResponse";
	 
	/**
	* Variable: errorMessage
	* Contains an error message where <status> != <Flashr.STATUS_OK>
	**/
	var errorMessage:String;
	
	/**
	* Variable: data
	* A plain object used to pass the additional data that is required to decypher responses.
	**/
	var data:Object;
	
	/**
	* Function: FlashrResponseEvent
	* Constructor
	**/
	function FlashrResponseEvent(request:FlashrRequest)
	{
		this.request = request;
		data = {};
	}
	
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.core.FlashrResponseEvent]";
	}
}