import com.kelvinluck.flashr.core.FlashrResponseEvent;
/**
* Class: com.kelvinluck.flashr.core.FlashrResponseHistory
* Stores information about completed Flickr API requests. Currently just wraps an Array
* but can potentially have methods such as getLastRequest(methodName) and suchlike added
* at a later date.
* 
* See Also:
* <Flashr.history>
* 
* <Flashr.saveHistory>
*
* Licence:
* This code and documentation is licensed under a Creative Commons 
* Attribution-NonCommercial-ShareAlike 2.0.
* <http://creativecommons.org/licenses/by-nc-sa/2.0/>
* 
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.core.FlashrResponseHistory 
{
	private static var _instance:FlashrResponseHistory;
	
	private var history:Array;
	
	/**
	* Function: FlashrRequestHistory
	* Private constructor, singleton.
	**/
	private function FlashrResponseHistory()
	{
		history = [];
	}
	
	/**
	* Function: addResponse
	* Adds a <FlashrResponseEvent> to the history.
	* 
	* Parameters:
	* response					-	The <FlashrResponseEvent> you want to add.
	**/
	function addResponse(response:FlashrResponseEvent)
	{
		history.push(response);
	}
	
	/**
	* Function: getHistory
	* Get's the current contents of the <history> Array.
	* 
	* Returns:
	* The Array containing all completed <FlashrResponseEvent>s.
	**/
	function getHistory():Array
	{
		return history;
	}
	
	
	/**
	* Function: getInstance
	* Gets a reference to the Singleton instance of this class.
	* 
	* Returns:
	* A reference to the one and only <FlashrRequestHistory>.
	**/
	public static function getInstance():FlashrResponseHistory
	{
		if (_instance == null) {
			_instance = new FlashrResponseHistory();
		}
		return _instance;
	}
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.core.FlashrResponseHistory - "+history.length+" items]";
	}
}