import com.kelvinluck.util.LogWrapper;
import com.kelvinluck.flashr.core.FlashrResponse;
import com.dynamicflash.utils.Delegate;
import com.kelvinluck.flashr.core.ResultsSet;
import com.kelvinluck.flashr.core.Photo;
import com.kelvinluck.flashr.core.Flashr;
/**
* Class: FlashrSimpleSearch
*
* Very simple example showing how to search for a tag using Flashr 0.5 and then
* get more detailed information about each matching photo. 
* 
* Currently just traces the relvant information out.
*
* Author:
* Kelvin Luck
*/
class com.kelvinluck.flashr.example.FlashrSimpleSearch extends MovieClip
{
	
	public static var SEARCH_TAG:String = "snow";
	public static var NUM_RESULTS:Number = 10;
	
	private var _flashr:Flashr;
	private var _flashrResponse:FlashrResponse;
	
	private var _numResults:Number;
	private var _photos:Array;
	
	/**
	* Function: FlashrSimpleSearch
	* Constructor
	**/
	function FlashrSimpleSearch()
	{
		// uncomment the following lines if you want to see the internals of what is happening with Flashr.
		//LogWrapper.getInstance().init();
		//LogWrapper.getInstance().addTracePublisher();
		
		_photos = [];
		
		_flashr = Flashr.getFlashr();
		_flashr.apiKey = "b40e05adf210ad4c4cc4da00f99f4184";
		_flashr.cacheQueries = true;
		
		
		_flashrResponse = new FlashrResponse();
		_flashrResponse.onPhotosSearch = Delegate.create(this, onPhotosSearch);
		_flashrResponse.onPhotosGetInfo = Delegate.create(this, onPhotosGetInfo);
		
		trace("Searching for " + NUM_RESULTS + " photos matching tag '" + SEARCH_TAG + "'");
		_flashr.photosSearch({tags:SEARCH_TAG, per_page:NUM_RESULTS});
	}
	
	function onPhotosSearch(rs:ResultsSet)
	{
		_numResults = rs.photos.length;
		trace("Loaded data for " + _numResults + " photos out of " + rs.total + " matching tag, requesting more details about each photo");
		for (var i:Number=0; i<_numResults; i++) {
			var thisPhoto:Photo = rs.photos[i];
			_flashr.photosGetInfo(thisPhoto.id, thisPhoto.secret);
		}
	}
	function onPhotosGetInfo(photo:Photo)
	{
		trace("Loaded detailed data for photo '" + photo.id + "'");
		_photos.push(photo);
		if (_photos.length == _numResults) {
			onAllPhotosLoaded();
		}
	}
	function onAllPhotosLoaded()
	{
		trace("*************ALL PHOTO INFO LOADED*************");
		trace(" ");
		for (var i:Number=0; i<_photos.length; i++) {
			var thisPhoto:Photo = _photos[i];
			trace("Photo " + thisPhoto.id + " :");
			trace("  Title: " + thisPhoto.title);
			trace("  Description: " + thisPhoto.description);
			trace("  Taken: " + thisPhoto.dateTaken);
			trace("  Author: " + thisPhoto.owner.username);
			trace("  Author ID: " + thisPhoto.owner.nsid);
			trace("  Thumbnail: " + thisPhoto.thumbnailUrl);
			trace("  Photo page URL: " + thisPhoto.photoPageUrl);
			trace("  Tags: " + thisPhoto.getTagsAsStrings());
			trace("------------------");
			trace(" ");
		}
	}
	
	/**
	* Function: toString
	**/
	public function toString():String
	{
		return "[com.kelvinluck.flashr.example.FlashrSimpleSearch]";
	}
}